@@ -1,6 +1,6 @@
 
 /*
- *           PVM 3.2:  Parallel Virtual Machine System 3.2
+ *         PVM version 3.3:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
@@ -22,8 +22,8 @@
  * purpose.  This software is provided ``as is'' without express or
  * implied warranty.
  *
- * PVM 3.2 was funded in part by the U.S. Department of Energy, the
- * National Science Foundation and the State of Tennessee.
+ * PVM version 3 was funded in part by the U.S. Department of Energy,
+ * the National Science Foundation and the State of Tennessee.
  */
 
 /*
@@ -32,6 +32,12 @@
  *	PVM console commands.
  *
 $Log: cmds.c,v $
+ * Revision 1.3  1994/10/15  18:38:56  manchek
+ * updated task flags and ps list headers
+ *
+ * Revision 1.2  1994/06/03  20:01:51  manchek
+ * version 3.3.0
+ *
  * Revision 1.1  1993/08/30  23:30:32  manchek
  * Initial revision
  *
@@ -46,12 +52,14 @@ $Log: cmds.c,v $
 #endif
 #include <ctype.h>
 #include <signal.h>
-#include "pvm3.h"
+#include <pvm3.h>
+#include <pvmtev.h>
 #include "cmd.h"
 #include "myalloc.h"
 #include "../src/listmac.h"
 #include "../src/bfunc.h"
 #include "job.h"
+#include "tevfmt.h"
 
 #define	PVMERRMSG(n)	((n) <= 0 && (n) > -pvm_nerr \
 						? pvm_errlist[-(n)] : "Unknown Error")
@@ -86,16 +94,19 @@ int spawn_cmd();
 int start_cmd();
 int tasks_cmd();
 int tickle_cmd();
+int trace_cmd();
 int unalias_cmd();
 int version_cmd();
 
 extern struct alias *aliases;			/* from cons.c */
-extern struct hostinfo *hostlist;		/* from cons.c */
+extern struct pvmhostinfo *hostlist;	/* from cons.c */
 extern struct job *joblist;				/* from job.c */
 extern int mytid;						/* from cons.c */
 extern int narchs;						/* from cons.c */
 extern int nextjob;						/* from cons.c */
 extern int nhosts;						/* from cons.c */
+extern struct tevfmt tev_formats[];		/* from tevfmt.c */
+
 
 struct cmdsw commands[] = {
 	{ "add",     2, 0, add_cmd },
@@ -119,13 +130,14 @@ struct cmdsw commands[] = {
 	{ "setenv",  1, 0, setenv_cmd },
 	{ "sig",     3, 0, sig_cmd },
 	{ "spawn",   2, 0, spawn_cmd },
+	{ "trace",   1, 0, trace_cmd },
 	{ "tickle",  2, 11, tickle_cmd },
-	{ "unalias", 2, 2, unalias_cmd },
+	{ "unalias", 2, 0, unalias_cmd },
 	{ "version", 1, 1, version_cmd },
 	{ 0, 0, 0, 0 }
 };
 
-static char rcsid[] = "$Id: cmds.c,v 1.1 1993/08/30 23:30:32 manchek Exp $";
+static char rcsid[] = "$Id: cmds.c,v 1.3 1994/10/15 18:38:56 manchek Exp $";
 
 
 freealias(ap)
@@ -147,7 +159,7 @@ newalias(name, num, args)
 	int num;
 	char **args;
 {
-	struct alias *ap;
+	struct alias *ap, *ap2;
 
 	ap = TALLOC(1, struct alias, "alias");
 	ap->a_name = STRALLOC(name);
@@ -155,7 +167,10 @@ newalias(name, num, args)
 	ap->a_num = num;
 	while (num-- > 0)
 		ap->a_args[num] = STRALLOC(args[num]);
-	LISTPUTBEFORE(aliases, ap, a_link, a_rlink);
+	for (ap2 = aliases->a_link; ap2 != aliases; ap2 = ap2->a_link)
+		if (strcmp(ap2->a_name, name) > 0)
+			break;
+	LISTPUTBEFORE(ap2, ap, a_link, a_rlink);
 	return ap;
 }
 
@@ -182,6 +197,7 @@ docmd(cmd)
 	int i;
 	int ac;
 	char *av[128];
+	int self;
 
 	/*
 	* parse command
@@ -217,8 +233,11 @@ docmd(cmd)
 				av[i + ap->a_num - 1] = av[i];
 			ac += ap->a_num - 1;
 		}
+		self = !strcmp(av[0], ap->a_args[0]);
 		for (i = ap->a_num; i-- > 0; )
 			av[i] = ap->a_args[i];
+		if (self)
+			break;
 	}
 
 	/*
@@ -259,10 +278,9 @@ add_cmd(ac, av)
 	ac--;
 	sv = TALLOC(ac, int, "int");
 	if ((cc = pvm_addhosts(av, ac, sv)) >= 0) {
-		if (cc > 0) {
-			pvm_recv(-1, 99);	/* waste the notify message */
-			pvm_notify(PvmHostAdd, 99, 0, (int*)0);
-		}
+		if (cc > 0)
+			pvm_recv(-1, 0);	/* waste the notify message */
+
 		printf("%d successful\n", cc);
 		fputs("                    HOST     DTID\n", stdout);
 		for (i = 0; i < ac; i++)
@@ -395,7 +413,6 @@ static char *helptx[] = {
 	"conf   HOST    host name",
 	"conf   DTID    tid base of pvmd",
 	"conf   ARCH    xhost architecture",
-	"conf   MTU     UDP max packet size",
 	"conf   SPEED   host relative speed",
 
 	"delete DELETE - Delete hosts from virtual machine",
@@ -414,7 +431,7 @@ static char *helptx[] = {
 	"id Syntax:  id",
 
 	"insert-INSERT - Add entry to database",
-	"insert-Syntax:  insert index|-1 hexvalue",
+	"insert-Syntax:  insert name index|-1 hexvalue",
 
 	"jobs JOBS   - Display list of running jobs",
 	"jobs Syntax:  jobs [ options ]",
@@ -425,7 +442,7 @@ static char *helptx[] = {
 	"kill Options:  -c   kill children of tid",
 
 	"lookup-LOOKUP - Find entry in database",
-	"lookup-Syntax:  lookup index|-1",
+	"lookup-Syntax:  lookup name index|-1",
 
 	"mstat MSTAT  - Show status of hosts",
 	"mstat Syntax:  mstat name ...",
@@ -435,17 +452,23 @@ static char *helptx[] = {
 	"ps Options:  -a       all hosts (default is local)",
 	"ps           -hhost   specific host tid",
 	"ps           -nhost   specific host name",
+	"ps           -l       long (show process id)",
 	"ps           -x       show console task (default is not)",
 	"ps Output fields:",
 	"ps   HOST    host name",
 	"ps   A.OUT   executable name",
 	"ps   TID     task id",
 	"ps   PTID    parent task id",
+	"ps   PTID    task process id",
 	"ps   FLAG    status",
 	"ps FLAG values:",
+	"ps   f   task process is child of pvmd",
 	"ps   c   task connected to pvmd",
 	"ps   a   task waiting authorization",
 	"ps   o   task connection being closed",
+	"ps   H   hoster task",
+	"ps   R   resource manager task",
+	"ps   T   tasker task",
 
 	"pstat PSTAT  - Show status of tasks",
 	"pstat Syntax:  pstat tid ...",
@@ -454,7 +477,7 @@ static char *helptx[] = {
 	"quit Syntax:  quit",
 
 	"remove-REMOVE - Delete entry from database",
-	"remove-Syntax:  remove index",
+	"remove-Syntax:  remove name index",
 
 	"reset RESET  - Kill all tasks",
 	"reset Syntax:  reset",
@@ -474,6 +497,8 @@ static char *helptx[] = {
 	"spawn           ->        redirect output of job to console",
 	"spawn           ->(file)  redirect output of job to file",
 	"spawn           ->>(file) append output of job to file",
+	"spawn           -@        trace job, display output on console",
+	"spawn           -@(file)  trace job, output to file",
 
 	"tickle-TICKLE - Tickle pvmd",
 	"tickle-Syntax:  tickle how [ arg ... ]",
@@ -489,8 +514,15 @@ static char *helptx[] = {
 	"tickle-  8   (dtid) trigger hostfail",
 	"tickle-  9   (rst) dump pvmd statistics, clear if rst true",
 
+	"trace TRACE  - Set/display trace event mask",
+	"trace Syntax:  trace",
+	"trace          trace [+] name ...",
+	"trace          trace - name ...",
+	"trace          trace [+] *",
+	"trace          trace - *",
+
 	"unalias UNALIAS - Undefine command alias",
-	"unalias Syntax:  unalias name",
+	"unalias Syntax:  unalias name ...",
 
 	"version VERSION - Show libpvm version",
 	"version Syntax:  version",
@@ -571,28 +603,31 @@ jobs_cmd(ac, av)
 	struct obuf *op;
 	int l = 0;
 	int ntask;
-	struct taskinfo *tip;
+	struct pvmtaskinfo *tip;
 
 	if (ac > 1 && !strcmp(av[1], "-l")) {
 		l = 1;
-		print_task_hdr();
+		print_task_hdr(0);
 		if (!hostlist)
 			pvm_config(&nhosts, &narchs, &hostlist);
 	}
 
 	for (jp = joblist->j_link; jp != joblist; jp = jp->j_link) {
-		printf("%d:%s", jp->j_jid, (l ? "\n" : ""));
-		for (op = jp->j_obufs->o_link; op != jp->j_obufs; op = op->o_link)
-			if (l) {
-				if (!pvm_tasks(op->o_tid, &ntask, &tip) && ntask == 1) {
-					print_task_rec(&tip[0]);
-				} else {
+		printf("%c%d:%s", (jp->j_flag & JOB_TRACE ? 'T' : ' '),
+				jp->j_jid, (l ? "\n" : ""));
+		if (jp->j_obufs) {
+			for (op = jp->j_obufs->o_link; op != jp->j_obufs; op = op->o_link)
+				if (l) {
+					if (!pvm_tasks(op->o_tid, &ntask, &tip) && ntask == 1)
+						print_task_rec(&tip[0], 0);
+					else
+						printf(" t%x", op->o_tid);
+					printf("\n");
+				} else
 					printf(" t%x", op->o_tid);
-				}
-
-			} else
-				printf(" t%x", op->o_tid);
-		printf("\n");
+			if (!l)
+				printf("\n");
+		}
 	}
 	return 0;
 }
@@ -607,7 +642,7 @@ kill_cmd(ac, av)
 	char *p;
 	int host = 0;
 	int cflg = 0;
-	struct taskinfo *tip;
+	struct pvmtaskinfo *tip;
 	int ntask;
 
 	if (ac > 1 && av[1][0] == '-') {
@@ -733,7 +768,7 @@ reset_cmd(ac, av)
 	int ac;
 	char **av;
 {
-	struct taskinfo *tip;
+	struct pvmtaskinfo *tip;
 	int ntask;
 	int i;
 	int j;
@@ -821,9 +856,11 @@ spawn_cmd(ac, av)
 	int count = 1;
 	int i;
 	int oflg = 0;
+	int tflg = 0;
 	int app;
 	char *ofn = 0;
-	struct job *jp;
+	char *tfn = 0;
+	struct job *jp, *jp2;
 
 	while (av[1][0] == '-') {
 		if (ac < 3) {
@@ -861,6 +898,12 @@ spawn_cmd(ac, av)
 					(ofn ? ofn : "(console)"));
 */
 		}
+		if (av[1][1] == '@') {
+			tflg = 1;
+			tfn = av[1] + 2;
+			if (!*tfn)
+				tfn = 0;
+		}
 		av++;
 		ac--;
 	}
@@ -880,10 +923,31 @@ spawn_cmd(ac, av)
 	} else
 		pvm_setopt(PvmOutputTid, 0);
 
+	if (tflg) {
+		pvm_setopt(PvmTraceTid, mytid);
+		pvm_setopt(PvmTraceCode, nextjob);
+		jp2 = job_new(nextjob);
+		jp2->j_flag |= JOB_TRACE;
+		printf("[%d]\n", nextjob++);
+		if (tfn) {
+			jp2->j_ff = fopen(tfn, "w");
+			if (!jp2->j_ff) {
+				perror(tfn);
+				job_free(jp2);
+				return 1;
+			}
+		}
+
+	} else {
+		pvm_setopt(PvmTraceTid, 0);
+	}
+
 	tids = TALLOC(count > 1 ? count : 1, int, "int");
 	if ((i = pvm_spawn(av[1], &av[2], flags, where, count, tids)) >= 0) {
 		if (oflg & !i)
 			job_free(jp);
+		if (tflg & !i)
+			job_free(jp2);
 		printf("%d successful\n", i);
 		for (i = 0; i < count; i++)
 			if (tids[i] < 0)
@@ -897,8 +961,8 @@ spawn_cmd(ac, av)
 
 
 static char *tflgs[] = {
-	"?", "?", "c",
-	"a", "o",
+	0, "f", "c", "a", "o", 0, 0, 0,
+	"R", "H", "T"
 };
 
 char *
@@ -908,11 +972,11 @@ task_flags(f)
 	static char buf[64];
 	int bit, i;
 
-	sprintf(buf, "0x%02x/", f);
+	sprintf(buf, "%x/", f);
 	i = sizeof(tflgs)/sizeof(tflgs[0]) - 1;
 	bit = 1 << i;
 	while (i >= 0) {
-		if (f & bit) {
+		if ((f & bit) && tflgs[i]) {
 			strcat(buf, tflgs[i]);
 			strcat(buf, ",");
 		}
@@ -924,17 +988,23 @@ task_flags(f)
 }
 
 
-print_task_hdr()
+print_task_hdr(lflg)
+	int lflg;
 {
-	fputs("                    HOST        A.OUT      TID     PTID FLAG\n",
+	if (lflg)
+		fputs("                    HOST      TID     PTID    PID   FLAG 0x COMMAND\n",
+			stdout);
+	else
+		fputs("                    HOST      TID   FLAG 0x COMMAND\n",
 			stdout);
 }
 
 
-print_task_rec(tip)
-	struct taskinfo *tip;
+print_task_rec(tip, lflg)
+	struct pvmtaskinfo *tip;
+	int lflg;
 {
-	struct hostinfo *hip = 0;
+	struct pvmhostinfo *hip = 0;
 
 	if (hostlist) {
 		for (hip = hostlist + nhosts - 1; hip >= hostlist; hip--)
@@ -947,19 +1017,22 @@ print_task_rec(tip)
 		printf("%24s", hip->hi_name);
 	else
 		printf("%24x", tip->ti_host);
-	printf(" %12s", tip->ti_a_out[0] ? tip->ti_a_out : "-");
 	if (tip->ti_tid == mytid)
 		printf("   (cons)");
 	else
 		printf(" %8x", tip->ti_tid);
-	if (tip->ti_ptid == mytid)
-		printf("   (cons)");
-	else
-		if (tip->ti_ptid)
-			printf(" %8x", tip->ti_ptid);
+	if (lflg) {
+		if (tip->ti_ptid == mytid)
+			printf("   (cons)");
 		else
-			printf("        -");
-	printf(" %s\n", task_flags(tip->ti_flag));
+			if (tip->ti_ptid)
+				printf(" %8x", tip->ti_ptid);
+			else
+				printf("        -");
+		printf(" %6d", tip->ti_pid);
+	}
+	printf(" %9s", task_flags(tip->ti_flag));
+	printf(" %-12s", tip->ti_a_out[0] ? tip->ti_a_out : "-");
 	return 0;
 }
 
@@ -968,13 +1041,17 @@ tasks_cmd(ac, av)
 	int ac;
 	char **av;
 {
-	struct taskinfo *tip;
+	struct pvmtaskinfo *tip;
 	int ntask;
 	int i;
-	struct hostinfo *hip = 0;
+	struct pvmhostinfo *hip = 0;
 	int xflg = 0;
 	char *p;
 	int host = pvm_tidtohost(mytid);
+	int lflg = 0;
+
+	if (!hostlist)
+		pvm_config(&nhosts, &narchs, &hostlist);
 
 	if (ac > 1) {
 		for (p = av[1]; *p; p++)
@@ -982,13 +1059,17 @@ tasks_cmd(ac, av)
 
 			case 'n':
 				p++;
-				for (hip = hostlist + nhosts; --hip >= hostlist; )
-					if (!(strcmp(p, hip->hi_name))) {
-						host = hip->hi_tid;
-						break;
-					}
-				if (hip < hostlist)
-					printf("unknown host %s\n", p);
+				if (!strcmp(p, "."))
+					pvm_tidtohost(mytid);
+				else {
+					for (hip = hostlist + nhosts; --hip >= hostlist; )
+						if (!(strcmp(p, hip->hi_name))) {
+							host = hip->hi_tid;
+							break;
+						}
+					if (hip < hostlist)
+						printf("unknown host %s\n", p);
+				}
 				while (*p)
 					p++;
 				p--;
@@ -1005,6 +1086,10 @@ tasks_cmd(ac, av)
 				host = 0;
 				break;
 
+			case 'l':
+				lflg = 1;
+				break;
+
 			case 'x':
 				xflg = 1;
 				break;
@@ -1018,15 +1103,13 @@ tasks_cmd(ac, av)
 			}
 	}
 
-	if (!hostlist)
-		pvm_config(&nhosts, &narchs, &hostlist);
-
 	if (!pvm_tasks(host, &ntask, &tip)) {
-		print_task_hdr();
+		print_task_hdr(lflg);
 		for (i = 0; i < ntask; i++) {
 			if (tip[i].ti_tid == mytid && !xflg)
 				continue;
-			print_task_rec(&tip[i]);
+			print_task_rec(&tip[i], lflg);
+			printf("\n");
 		}
 	}
 }
@@ -1055,14 +1138,141 @@ tickle_cmd(ac, av)
 }
 
 
+int sortedtevs[] = {
+	0,   154, 2,   4,   6,
+	156, 8,   10,  12,  14,  16,  178, 18,
+	20,  158, 22,  24,  26,  28,  160, 30,
+	32,  162, 34,  36,  38,  40,  42,  44,
+	46,  48,  50,  52,  54,  56,  164, 58,
+	60,  62,  64,  66,  68,  70,  72,  76,
+	80,  84,  74,  78,  82,  166, 86,  168,
+	88,  90,  92,  170, 142, 144, 146, 180,
+	94,  96,  172, 98,  100, 102, 174, 104,
+	106, 108, 110, 112, 114, 176, 116, 118,
+	120, 122, 124, 126, 130, 134, 138, 128,
+	132, 136, 140, 150, 148, 152,
+};
+
+printtm(who)
+	int who;
+{
+	int maxt = sizeof(sortedtevs)/sizeof(sortedtevs[0]);
+/*
+	int maxt = TEV_MAX / 2 + 1;
+*/
+	int h[4];
+	int i[3];
+	int j;
+	int b;
+	int d;
+	int l;
+	char buf[64];
+	Pvmtmask tm;
+
+	pvm_gettmask(who, tm);
+
+	i[0] = h[0] = 0;
+	i[1] = h[1] = maxt / 3;
+	i[2] = h[2] = (maxt * 2) / 3;
+	h[3] = maxt;
+
+	do {
+		d = 0;
+		for (j = 0; j < 3; j++) {
+			if (i[j] < h[j + 1]) {
+				b = TEV_CHECK_MASK(tm, sortedtevs[i[j]]);
+				strcpy(buf, tev_formats[sortedtevs[i[j]]].name);
+/*
+				b = TEV_CHECK_MASK(tm, i[j] * 2);
+				strcpy(buf, tev_formats[i[j] * 2].name);
+*/
+				l = strlen(buf) - 1;
+				if (buf[l] == '0')
+					buf[l] = 0;
+				printf(" %c %-22s", (b ? '*' : ' '), buf);
+				i[j]++;
+			} else
+				printf("                         ");
+			if (i[j] >= h[j + 1])
+				d++;
+		}
+		printf("\n");
+	} while (d != 3);
+}
+
+
+trace_cmd(ac, av)
+	int ac;
+	char **av;
+{
+	int i;
+	int onoff = 1;
+	int e;
+	int l;
+	char buf[64];
+	Pvmtmask tm;
+
+	if (ac == 1) {
+		printtm(PvmTaskChild);
+		return 0;
+	}
+
+	pvm_gettmask(PvmTaskChild, tm);
+
+	for (i = 1; i < ac; i++) {
+		if (!strcmp(av[i], "+")) {
+			onoff = 1;
+		} else if (!strcmp(av[i], "-")) {
+			onoff = 0;
+		} else if (!strcmp(av[i], "*")) {
+			for (e = TEV_FIRST; e <= TEV_MAX; e += 2)
+				if (onoff)
+					TEV_SET_MASK(tm, e);
+				else
+					TEV_UNSET_MASK(tm, e);
+		} else {
+			if (strncmp(av[i], "pvm_", 4))
+				strcpy(buf, "pvm_");
+			else
+				buf[0] = 0;
+			strcat(buf, av[i]);
+			l = strlen(buf);
+/*
+			printf("<%s>\n", buf);
+*/
+			for (e = TEV_FIRST; e <= TEV_MAX; e += 2)
+				if (!strncmp(tev_formats[e].name, buf, l)
+				|| !strncmp(tev_formats[e].name, av[i], l))
+					break;
+			if (e <= TEV_MAX) {
+				if (onoff)
+					TEV_SET_MASK(tm, e);
+				else
+					TEV_UNSET_MASK(tm, e);
+
+			} else {
+				printf("no such event \"%s\"\n", av[i]);
+			}
+		}
+	}
+
+	pvm_settmask(PvmTaskChild, tm);
+
+	return 0;
+}
+
+
 unalias_cmd(ac, av)
 	int ac;
 	char **av;
 {
 	struct alias *ap;
+	int i;
 
-	if (ap = findalias(av[1]))
-		freealias(ap);
+	for (i = 1; i < ac; i++)
+		if (ap = findalias(av[i]))
+			freealias(ap);
+	return 0;
 }
 
 
