@@ -1,6 +1,6 @@
 
 /*
- *           PVM 3.2:  Parallel Virtual Machine System 3.2
+ *         PVM version 3.3:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
@@ -22,8 +22,8 @@
  * purpose.  This software is provided ``as is'' without express or
  * implied warranty.
  *
- * PVM 3.2 was funded in part by the U.S. Department of Energy, the
- * National Science Foundation and the State of Tennessee.
+ * PVM version 3 was funded in part by the U.S. Department of Energy,
+ * the National Science Foundation and the State of Tennessee.
  */
 
 /*
@@ -33,6 +33,16 @@
  *	machine and other tasks.
  *
 $Log: cons.c,v $
+ * Revision 1.4  1994/10/15  18:40:09  manchek
+ * don't compare add-host notify source to 0x80000000
+ *
+ * Revision 1.3  1994/06/03  20:01:52  manchek
+ * version 3.3.0
+ *
+ * Revision 1.2  1993/09/16  21:33:40  manchek
+ * moved notify(HostAdd) to before we read script file.
+ * added include for linux/time.h
+ *
  * Revision 1.1  1993/08/30  23:30:32  manchek
  * Initial revision
  *
@@ -40,7 +50,10 @@ $Log: cons.c,v $
 
 #include <stdio.h>
 #include <sys/types.h>
-#ifdef IMA_RS6K
+#ifdef IMA_LINUX
+#include <linux/time.h>
+#endif
+#if defined(IMA_RS6K) || defined(IMA_POWER4)
 #include <sys/select.h>
 #endif
 #ifdef	IMA_I860
@@ -48,7 +61,7 @@ $Log: cons.c,v $
 #endif
 #include <ctype.h>
 #include <signal.h>
-#include "pvm3.h"
+#include <pvm3.h>
 #include "cmd.h"
 #include "myalloc.h"
 #include "../src/listmac.h"
@@ -66,10 +79,10 @@ extern int pvm_nerr;
 int mytid = -1;
 int nhosts = 0;
 int narchs = 0;
-struct hostinfo *hostlist = 0;
+struct pvmhostinfo *hostlist = 0;
 struct alias *aliases = 0;
 
-static char rcsid[] = "$Id: cons.c,v 1.1 1993/08/30 23:30:32 manchek Exp $";
+static char rcsid[] = "$Id: cons.c,v 1.4 1994/10/15 18:40:09 manchek Exp $";
 
 
 main(argc, argv)
@@ -77,7 +90,7 @@ main(argc, argv)
 	char **argv;
 {
 	char *prompt = "pvm> ";
-	char cmd[512];
+	char cmd[1024];
 	int cc;
 	int i;
 	int src, cod, len;
@@ -87,14 +100,14 @@ main(argc, argv)
 	int nfds;
 	int n;
 
+	pvm_setopt(PvmResvTids, 1);
+
 	aliases = TALLOC(1, struct alias, "alias");
 	BZERO((char*)aliases, sizeof(struct alias));
 	aliases->a_link = aliases->a_rlink = aliases;
 
-/*
 	if (argc > 1 && argv[1][0] == '-' && argv[1][1] == 'd')
-		pvm_setdebug(xtoi(argv[1] + 2));
-*/
+		pvm_setopt(PvmDebugMask, xtoi(argv[1] + 2));
 
 	pvm_setopt(PvmRoute, PvmDontRoute);
 
@@ -119,6 +132,8 @@ main(argc, argv)
 /*
 	ttpcb_dumpall();
 */
+	pvm_notify(PvmHostAdd, 0, -1, (int*)0);
+
 	if (!(p = getenv("HOME")))
 		p = ".";
 	sprintf(cmd, "%s/.pvmrc", p);
@@ -138,8 +153,6 @@ main(argc, argv)
 		nfds = np[0] + 1;
 	}
 
-	pvm_notify(PvmHostAdd, 99, 0, (int*)0);
-
 	printf(prompt);
 	fflush(stdout);
 	while (1) {
@@ -311,8 +324,7 @@ checkmsgs()
 	while ((cc = pvm_nrecv(-1, -1)) > 0) {
 		pvm_bufinfo(cc, &len, &cod, &src);
 
-		if (cod == 99 && src == 0x80000000) {
-			pvm_notify(PvmHostAdd, 99, 0, (int*)0);
+		if (cod == 0) {
 			pvm_upkint(&n, 1, 1);
 			dtids = TALLOC(n, int, "");
 			pvm_upkint(dtids, n, 1);
@@ -337,7 +349,7 @@ checkmsgs()
 			continue;
 		}
 
-		if (cod >= 100 && (jp = job_find(cod))) {
+		if (cod > 0 && (jp = job_find(cod))) {
 			checkoutput(jp, cc, len, cod, src);
 			continue;
 		}
