@@ -1,25 +1,19 @@
 c-----------------------------------------------------------------------------
-c This Program tests all the PVM 3.0 Fortran calls.
+c This Program tests most of  the PVM 3 Fortran calls.
 c It should be executed from a Unix prompt, not from pvm console:
-c  % testall <RETURN>
+c 
 c Because the Fortran library is just an interface to the C routines
 c this program also tests all of the C routines required by 
 c the Fortran interface.
 c
-c The following C routine features are not available in Fortran 
-c   pvm_spawn    using argv
-c   pvm_config   returning an array of hostp structures
-c   pvm_addhosts adding multiple hosts
-c   pvm_delhosts deleting multiple hosts
-c   pvm_recvf    changing receive context
 c-----------------------------------------------------------------------------
       implicit none
       include '../include/fpvm3.h'
-      integer mytid, off, on, prev, parentid, nproc, nitem, info
+      integer mytid, parentid, nproc, nitem, info
       integer i, mbuf, oldbuf, err, stid, msgtag, bytes, ntask 
-      integer nhost, narch, mstat, pstat, nobuf, j
+      integer nhost, narch, nobuf
       integer tids(10), sbuf(3), rbuf(2)
-      integer htids(10), mtus(10), speeds(10)
+      integer htids(10), speeds(10)
       integer iwhich, ttids(10), ptids(10), flag(10)
       integer*2   i2data(100)
       integer*4   i4data(100)
@@ -31,11 +25,12 @@ c-----------------------------------------------------------------------------
       character*16  hosts(10)
       character*16  aout(10)
       character*8   archs(10)
+      character   buffer*20
 
-      print *,'Testing PVM 3.0 Fortran Routines'
+      print *,'Testing PVM 3 Fortran Routines'
       print *,' '
 c     -------------------
-c     Enroll into PVM 3.0
+c     Enroll into PVM 3
 c     -------------------
       CALL PVMFMYTID( mytid )
 
@@ -45,18 +40,6 @@ c     -------------------
       else
          print *,'My task id is ',mytid
       endif
-c     ---------------------------
-c     Turn on auto error checking
-c     ---------------------------
-c     off = 0
-      on  = 1
-      CALL PVMFSERROR( on, prev )
-      if( prev .lt. 0 ) then
-         print *,'Error in pvmfserror error=',prev
-      else
-         print *,'Activated PVM auto error checking'
-         print *,'Errors written to log file /tmp/pvml.nnnnn'
-      endif
 c     ---------------
 c     Test pvmfparent
 c     ---------------
@@ -167,26 +150,24 @@ c        -------------------------
          print *,' '
          print *,'Testing Information routines'
          CALL PVMFCONFIG( nhost, narch, htids(1), hosts(1), archs(1), 
-     >                    mtus(1), speeds(1), info )
+     >                    speeds(1), info )
          if( info .lt. 0 ) then
             print *,'Error in pvmfconfig info=',info
          else
             print *,'There are ',nhost, 'hosts' 
             print *,'The present configuration is:'
-            print *,'            hostname        ARCH ',
-     >              '       pvmd-tid   packet size       speed'
-            print *,'        -------------------------',
+            print *,'   hostname        ARCH ',
+     >              '   pvmd-tid   speed'
+            print *,' -------------------------',
      >              '-----------------------------------------'
             i = 1
-            print *,i, hosts(i), archs(i), htids(i),
-     >              mtus(i), speeds(i)
+            print *,i, hosts(i), archs(i), htids(i), speeds(i)
             do 123 i=2, nhost
                CALL PVMFCONFIG( nhost, narch, htids(i), hosts(i),
-     >                    archs(i), mtus(i), speeds(i), info )
-               print *,i, hosts(i), archs(i), htids(i),
-     >                 mtus(i), speeds(i)
+     >                    archs(i), speeds(i), info )
+               print *,i, hosts(i), archs(i), htids(i), speeds(i)
   123       continue
-            print *,'        -------------------------',
+            print *,' -------------------------',
      >              '-----------------------------------------'
          endif
          print *,' '
@@ -199,9 +180,9 @@ c        -------------------------
 
             print *,'There are ',ntask, 'tasks' 
             print *,'Running PVM tasks:'
-            print *,'            tasktname            tid ',
-     >              '     parent-tid    pvmd-tid      flags'
-            print *,'        -------------------------',
+            print *,'  tasktname            tid ',
+     >              '  parent-tid    pvmd-tid      flags'
+            print *,' -------------------------',
      >              '-----------------------------------------'
             i = 1
             print *,i, aout(i), ttids(i), ptids(i), htids(i), flag(i)
@@ -210,23 +191,11 @@ c        -------------------------
      >                    htids(i), flag(i), aout(i), info )
                print *,i, aout(i),ttids(i),ptids(i),htids(i),flag(i)
   234       continue
-            print *,'        -------------------------',
+            print *,' -------------------------',
      >              '-----------------------------------------'
          endif
          print *,' '
          print *,' other routines'
-         CALL PVMFMSTAT( host, mstat)
-         if( mstat .lt. 0 ) then
-            print *,'Error in pvmfmstat mstat=',mstat
-         else
-            print *,'pvmfmstat returns ',mstat
-         endif
-         CALL PVMFPSTAT( tids(nproc), pstat )
-         if( pstat .lt. 0 ) then
-            print *,'Error in pvmfpstat pstat=',pstat
-         else
-            print *,'pvmfpstat returns ',pstat
-         endif
 c        -----------------------------------------
 c        Test killing processes and deleting hosts
 c        -----------------------------------------
@@ -315,7 +284,7 @@ c        ------------------------------
             print *,'Error in pvmffreebuf (recv) error=', info
          endif
       endif      
-      CALL PVMFMKBUF( PVMINPLACE, sbuf(3) )
+      CALL PVMFMKBUF( PVMDATAINPLACE, sbuf(3) )
       if( sbuf(3) .lt. 1 ) then
          print *,'Error in pvmfmkbuf using inplace option =',sbuf(3)
       endif      
@@ -357,7 +326,7 @@ c     ----------------------
       CALL PVMFSEND( tids(2), 44, info )
       CALL PVMFRECV( tids(2), 44, info )
 
-      CALL PVMFUNPACK( STRING,  'Pack Test',  9, 1, info )
+      CALL PVMFUNPACK( STRING,    buffer,     9, 1, info )
       if( info .lt. 0 ) print *,'Error in pvmfunpack (STRING)'
       CALL PVMFUNPACK( BYTE1,     i4data,    64, 1, info )
       if( info .lt. 0 ) print *,'Error in pvmfunpack (BYTE1)'
@@ -444,7 +413,7 @@ c     --------------------------------------------
          CALL PVMFSEND( parentid, 65, info )
       else if( msgtag .eq. 44 ) then
          nitem = 100
-         CALL PVMFUNPACK( STRING,  'Pack Test',  9, 1, info )
+         CALL PVMFUNPACK( STRING,    buffer,     9, 1, info )
          CALL PVMFUNPACK( BYTE1,     i4data,    64, 1, info )
          CALL PVMFUNPACK( INTEGER2,  i2data, nitem, 1, info )
          CALL PVMFUNPACK( INTEGER4,  i4data, nitem, 1, info )
