@@ -1,6 +1,6 @@
 
 /*
- *           PVM 3.2:  Parallel Virtual Machine System 3.2
+ *         PVM version 3.3:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
@@ -22,8 +22,8 @@
  * purpose.  This software is provided ``as is'' without express or
  * implied warranty.
  *
- * PVM 3.2 was funded in part by the U.S. Department of Energy, the
- * National Science Foundation and the State of Tennessee.
+ * PVM version 3 was funded in part by the U.S. Department of Energy,
+ * the National Science Foundation and the State of Tennessee.
  */
 
 /*
@@ -92,6 +92,18 @@
  *      deleted loclinput(), and merged loclinpkt() into pvmd.c
  *
 $Log: pvmdmimd.c,v $
+ * Revision 1.4  1994/11/07  21:30:45  manchek
+ * Modify mpp_output() and mpp_mcast() to send a null packet to alert precv().
+ * Modify mpp_input() to handle DataInPlace properly.
+ * mpp_new() should return PvmOutOfRes when it runs out of nodes.
+ * Fix a bug in the way ptype is computed in mpp_mcast()
+ *
+ * Revision 1.3  1994/06/03  20:54:52  manchek
+ * version 3.3.0
+ *
+ * Revision 1.2  1993/12/20  15:39:49  manchek
+ * patch 6 from wcj
+ *
  * Revision 1.1  1993/08/30  23:36:09  manchek
  * Initial revision
  *
@@ -124,6 +136,7 @@ $Log: pvmdmimd.c,v $
 #include "pkt.h"
 #include "task.h"
 #include "listmac.h"
+#include "pvmdmp.h"
 #include "pvmmimd.h"
 #include "bfunc.h"
 
@@ -141,7 +154,6 @@ extern int myhostpart;			/* from pvmd.c */
 extern int myndf;				/* from pvmd.c */
 extern struct htab *hosts;		/* from pvmd.c */
 extern int tidhmask;			/* from pvmd.c */
-extern struct task *locltasks;	/* from task.c */
 extern int ourudpmtu;			/* from pvmd.c */
 
 int partsize;					/* size of partition */
@@ -150,7 +162,7 @@ int tidnmask = TIDNODE;			/* mask for node field of tids */
 
 /* private */
 
-static char rcsid[] = "$Id: pvmdmimd.c,v 1.1 1993/08/30 23:36:09 manchek Exp $";
+static char rcsid[] = "$Id: pvmdmimd.c,v 1.4 1994/11/07 21:30:45 manchek Exp $";
 static struct nodeset *busynodes;	/* active nodes; ordered by proc type */
 static char etext[512];			/* scratch for error log */
 static int ptypemask;			/* mask; we use these bits of ptype in tids */
@@ -227,6 +239,8 @@ ptype = 0;
 			}
 		}
 	}
+
+#if 0
 	if ((sp = busynodes->n_rlink) != busynodes) {
 		last = -1;
 		ptype = sp->n_ptype + 1;
@@ -240,6 +254,12 @@ ptype = 0;
 			goto done;		
 	}
 	pvmlogerror("mpp_new() out of descriptors: too many spawns\n");
+#endif /*0*/
+
+	if (partsize - last <= count) {
+		pvmlogerror("mpp_new() not enough nodes in partition\n");
+		return (struct nodeset *)0;
+	}
 
 done:
 	if (debugmask & PDMNODE) {
@@ -336,7 +356,10 @@ mpp_load(flags, name, argv, count, tids, ptid)
         }
 
 		pids = TALLOC(partsize, long, "pids");
-		sp = mpp_new(count, ptid);
+		if (!(sp = mpp_new(count, ptid))) {
+			err = PvmOutOfRes;
+			goto done;
+		}
 		nnodes = (count == partsize) ? -1 : count;
 		nodes = TALLOC(count, long, "nodes");
 		for (j = 0; j < count; j++)
@@ -362,10 +385,9 @@ mpp_load(flags, name, argv, count, tids, ptid)
 		pvminfo[2] = ptid;
 		pvminfo[3] = MAXFRAGSIZE;
 		pvminfo[4] = myndf;
-/*
-		if (count == -1)
+		if (count == partsize)
 			err = _csend(PMTHOST, pvminfo, sizeof(pvminfo), -1, sp->n_ptype);
-*/
+		else {
 		if (sp->n_ptype != PVMDPTYPE && _setptype(sp->n_ptype) < 0)
 			pvmlogperror("mpp_load() setptype");
 		if (_gsendx(PMTHOST, pvminfo, sizeof(pvminfo), nodes, (long)count)
@@ -376,11 +398,7 @@ mpp_load(flags, name, argv, count, tids, ptid)
 		}
 		if (sp->n_ptype != PVMDPTYPE)
 			_setptype(PVMDPTYPE);
-
-/*
-cprobe(-1);
-mpp_input();
-*/
+		}
 
 		if (nodes)
 			PVM_FREE(nodes);
@@ -422,6 +440,8 @@ pvmlogerror(etext);
 done:
 	for (j = 0; j < count; j++)
 		tids[j] = err;
+	if (pids)
+		PVM_FREE(pids);
 	return err;
 }
 
@@ -444,15 +464,33 @@ mpp_input()
 	pp->pk_dat += hdrdiff;
 	if (_crecv(-1, pp->pk_dat, (long)len) < 0)
 		pvmlogperror("mpp_input() crecv");
+	else {
+		cp = pp->pk_dat;
+		if ((len = pvmget32(cp + 8) + TDFRAGHDR) != pp->pk_len) {
+			struct pkt *hdr;
 
-	loclinpkt(0, pp);
+			/* inplace frag, head & body sent separately */
+			hdr = pp;
+			pp = pk_new(len + hdrdiff);
+			pp->pk_dat += hdrdiff;
+			BCOPY(hdr->pk_dat, pp->pk_dat, hdr->pk_len);
+			pp->pk_len = len;
+			if (_crecvx(PMTPACK, pp->pk_dat + hdr->pk_len, 
+			(long)(len - hdr->pk_len), _infonode(), 0, (long*)0) < 0) {
+				pvmlogperror("mpp_input() crecv body");
+				return;
+			}
+			pk_free(hdr);
+		}
+		loclinpkt(0, pp);
+	}
 }
 
 
 /* use gsendx() to multicast */
+/* XXX can't multicast to tasks with different ptypes */
 int
-mpp_mcast(src, pp, tids, ntask)
-	int src;		/* sender's tid */
+mpp_mcast(pp, tids, ntask)
 	struct pkt *pp;	/* packet to send */
 	int tids[];		/* target tasks */
 	int ntask;		/* how many */
@@ -460,32 +498,63 @@ mpp_mcast(src, pp, tids, ntask)
 	int i;
 	long count = 0;
 	struct nodeset *sp;
-	long ptype;
+	long ptype = -1;
 	int ptypepart;
 	long *nodes = 0;
 	char *cp;
 
-	ptype = TIDTOTYPE(tids[0]);
+	nodes = TALLOC(ntask, long, "nodes");
+	for (i = 0; i < ntask; i++) {
+/*
+		if ((tids[i] & tidtmask) != ptypepart 
+				|| (tids[i] & tidhmask) != myhostpart)
+*/
+		if ((tids[i] & tidhmask) != myhostpart)
+			continue;
+        if (!TIDISNODE(tids[i])) {      /* Arrrgh!!! */
+            struct pkt *pp2;
+            struct task *tp;
+
+            if (tp = task_find(tids[i])) {
+                pp2 = pk_new(0);
+                pp2->pk_src = pp->pk_src;
+                pp2->pk_dst = pp->pk_dst;
+                pp2->pk_flag = pp->pk_flag;
+                pp2->pk_buf = pp->pk_buf;
+                pp2->pk_max = pp->pk_max;
+                pp2->pk_dat = pp->pk_dat;
+                pp2->pk_len = pp->pk_len;
+                da_ref(pp->pk_buf);
+                pkt_to_task(tp, pp2);
+            }
+            continue;
+        } 
+		if (ptype == -1) {
+			ptype = TIDTOTYPE(tids[i]);
+			ptypepart = tids[i] & tidtmask;
+		}
+		if (ptype != TIDTOTYPE(tids[i]))
+			pvmlogerror("mpp_mcast(): multiple ptypes? Oh no ..\n");
+		nodes[count++] = tids[i] & tidnmask;
+	}
 	for (sp = busynodes->n_link; sp != busynodes; sp = sp->n_link)
 		if ((sp->n_ptype & ptypemask) == ptype) {
 			ptype = sp->n_ptype;
 			break;
 		}
 	if (sp == busynodes) {
-		sprintf(etext, "mpp_mcast() pkt from t%x scrapped (no cube)\n", src);
+		sprintf(etext, "mpp_mcast() pkt from t%x scrapped (no cube)\n", 
+		pp->pk_src);
 		pvmlogerror(etext);
 		return -1;
 	}
 
-	ptypepart = tids[0] & tidtmask;
-	nodes = TALLOC(ntask, long, "nodes");
-	for (i = 0; i < ntask; i++) {
-		if ((tids[i] & tidtmask) != ptypepart 
-				|| (tids[i] & tidhmask) != myhostpart)
-			continue;
-		nodes[count++] = tids[i] & tidnmask;
-	}
 	if (count) {
+		if (debugmask & PDMMESSAGE) {
+			sprintf(etext, "mpp_mcast() pkt from t%x len=%d to type=%ld %ld\n",
+					pp->pk_src, pp->pk_len, ptype, count);
+			pvmlogerror(etext);
+		}
 		cp = (pp->pk_dat -= TDFRAGHDR);
         pvmput32(cp, pp->pk_dst);
         pvmput32(cp + 4, pp->pk_src);
@@ -494,17 +563,15 @@ mpp_mcast(src, pp, tids, ntask)
         pp->pk_len += TDFRAGHDR;
 		if (ptype != PVMDPTYPE && _setptype(ptype) < 0)
 			pvmlogperror("mpp_mcast() setptype");
-		if (_gsendx(PMTMCAST, pp->pk_dat, (long)pp->pk_len, nodes, count) < 0)
+		/* inform precv/recv of a packed message */
+		if (_gsendx(PMTPACK, cp, 0, nodes, count) < 0)
+			pvmlogperror("mpp_mcast() gsendx null msg");
+		if (_gsendx(PMTPACK, pp->pk_dat, (long)pp->pk_len, nodes, count) < 0)
 			pvmlogperror("mpp_mcast() gsendx");
 		if (ptype != PVMDPTYPE)
 			_setptype(PVMDPTYPE);
-		if (debugmask & PDMMESSAGE) {
-			sprintf(etext, "mpp_mcast() pkt from t%x len=%d to type=%ld %ld\n",
-					src, pp->pk_len, ptype, count);
-			pvmlogerror(etext);
-		}
-		PVM_FREE(nodes);
 	}
+	PVM_FREE(nodes);
 	return 0;
 }
 
@@ -537,7 +604,7 @@ mpp_output(tp, pp)
 
 	if (debugmask & PDMPACKET) {
 		sprintf(etext,
-			"mpp_output() src t%x dst t%x ff %x len %d ptype=%ld\n",
+			"mpp_output() src t%x dst t%x ff %x len %d ptype %d\n",
 			pp->pk_src, pp->pk_dst, pp->pk_flag & (FFSOM|FFEOM),
 			pp->pk_len, ptype);
 		pvmlogerror(etext);
@@ -556,7 +623,12 @@ mpp_output(tp, pp)
 			outpkt = 0;
 		}
 	}
-	if ((isendmid = _isend(PMTDN, cp, (long)pp->pk_len, node, ptype)) < 0) {
+	/* inform precv/recv of a packed message */
+	if (_csend(PMTPACK, cp, 0, node, ptype) < 0) {
+		sprintf(etext,"mpp_output() can't send null msg to t%x", tp->t_tid);
+		pvmlogperror(etext);
+	}
+	if ((isendmid = _isend(PMTPACK, cp, (long)pp->pk_len, node, ptype)) < 0) {
 		sprintf(etext,"mpp_output() can't send to t%x", tp->t_tid);
 		pvmlogperror(etext);
 		pk_free(pp);
@@ -590,6 +662,7 @@ mpp_probe()
 
 
 /* break pkt into smaller fragments and put them on txq */
+/*
 pkt_to_task(tp, pp)
 	struct task *tp;
 	struct pkt *pp;
@@ -622,3 +695,4 @@ pkt_to_task(tp, pp)
 	pk_free(pp);
 	return 0;
 }
+*/
