@@ -1,6 +1,6 @@
 
 /*
- *           PVM 3.2:  Parallel Virtual Machine System 3.2
+ *         PVM version 3.3:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
@@ -22,8 +22,8 @@
  * purpose.  This software is provided ``as is'' without express or
  * implied warranty.
  *
- * PVM 3.2 was funded in part by the U.S. Department of Energy, the
- * National Science Foundation and the State of Tennessee.
+ * PVM version 3 was funded in part by the U.S. Department of Energy,
+ * the National Science Foundation and the State of Tennessee.
  */
 
 /*
@@ -32,6 +32,12 @@
  *	Pvmd-pvmd protocol
  *
 $Log: ddpro.h,v $
+ * Revision 1.3  1994/07/18  19:19:27  manchek
+ * added PDMWAITC
+ *
+ * Revision 1.2  1994/06/03  20:38:13  manchek
+ * version 3.3.0
+ *
  * Revision 1.1  1993/08/30  23:26:47  manchek
  * Initial revision
  *
@@ -41,7 +47,7 @@ $Log: ddpro.h,v $
 /* protocol version */
 
 #ifndef	DDPROTOCOL
-#define	DDPROTOCOL	2312
+#define	DDPROTOCOL	2315
 #endif
 
 #define	DDMINRETRIES	10
@@ -57,7 +63,7 @@ $Log: ddpro.h,v $
 /* message/frag hdr sizes */
 
 #define	DDFRAGHDR		16
-#define	DDMSGHDR		8
+#define	DDMSGHDR		16
 
 /* frag flags */
 
@@ -67,36 +73,42 @@ $Log: ddpro.h,v $
 
 /* d-d message flavors */
 
-#define	DM_ADDHOST		1		/* add hosts to machine */
-#define	DM_ADDHOSTACK	2
-#define	DM_SPAWN		3		/* spawn tasks */
-#define	DM_SPAWNACK		4
-#define	DM_SENDSIG		5		/* signal a task */
-#define	DM_HTUPD		6		/* define new host table */
-#define	DM_HTUPDACK		7
-#define	DM_HTCOMMIT		8		/* switch to new host table */
-#define	DM_SLCONF		9		/* config slave pvmd */
-#define	DM_STARTACK		10		/* pvmd' completed slave startup */
-#define	DM_TASK			11		/* get list of tasks */
-#define	DM_TASKACK		12
-#define	DM_DELHOST		13		/* delete hosts from machine */
-#define	DM_DELHOSTACK	14
-#define	DM_NULL			15		/* null message */
-#define	DM_TASKOUT		16		/* task standard out/err data */
-#define	DM_PSTAT		17		/* query status of task */
-#define	DM_PSTATACK		18
-#define	DM_HALT			19		/* request master pvmd to halt */
-#define	DM_MCA			20		/* define multicast tid */
-#define	DM_NOTIFY		21		/* request notify */
-#define	DM_NOTIFYACK	22
-#define	DM_DB			23		/* name-server request */
-#define	DM_DBACK		24
-#define	DM_HTDEL		25		/* delete hosts from host table */
+#define	DM_FIRST		0x80020001		/* first DM_ message */
+#define	DM_ADD			(DM_FIRST+0)	/* add hosts to machine */
+#define	DM_ADDACK		(DM_FIRST+1)
+#define	DM_EXEC			(DM_FIRST+2)	/* start tasks */
+#define	DM_EXECACK		(DM_FIRST+3)
+#define	DM_SENDSIG		(DM_FIRST+4)	/* signal a task */
+#define	DM_HTUPD		(DM_FIRST+5)	/* define new host table */
+#define	DM_HTUPDACK		(DM_FIRST+6)
+#define	DM_HTCOMMIT		(DM_FIRST+7)	/* switch to new host table */
+#define	DM_SLCONF		(DM_FIRST+8)	/* config slave pvmd */
+#define	DM_STARTACK		(DM_FIRST+9)	/* pvmd' completed slave startup */
+#define	DM_TASK			(DM_FIRST+10)	/* get list of tasks */
+#define	DM_TASKACK		(DM_FIRST+11)
+#define	DM_DELHOST		(DM_FIRST+12)	/* delete hosts from machine */
+#define	DM_DELHOSTACK	(DM_FIRST+13)
+#define	DM_NULL			(DM_FIRST+14)	/* null message */
+#define	DM_TASKOUT		(DM_FIRST+15)	/* task standard out/err data */
+#define	DM_PSTAT		(DM_FIRST+16)	/* query status of task */
+#define	DM_PSTATACK		(DM_FIRST+17)
+#define	DM_HALT			(DM_FIRST+18)	/* request master pvmd to halt */
+#define	DM_MCA			(DM_FIRST+19)	/* define multicast tid */
+#define	DM_NOTIFY		(DM_FIRST+20)	/* request notify */
+#define	DM_NOTIFYACK	(DM_FIRST+21)
+#define	DM_DB			(DM_FIRST+22)	/* name-server request */
+#define	DM_DBACK		(DM_FIRST+23)
+#define	DM_HTDEL		(DM_FIRST+24)	/* delete hosts from host table */
+#define	DM_HOSTSYNC		(DM_FIRST+25)	/* request clock sample */
+#define	DM_HOSTSYNCACK	(DM_FIRST+26)
+#define	DM_LAST			(DM_FIRST+26)	/* last DM_ message */
 
 /* DM_SLCONF fields - don't use 0 */
 
 #define	DM_SLCONF_EP	1		/* set epath */
 #define	DM_SLCONF_BP	2		/* set bpath */
+#define	DM_SLCONF_WD	3		/* set wdir */
+#define	DM_SLCONF_SCHED	4		/* set scheduler tid */
 
 /* pvmd debug mask */
 
@@ -107,6 +119,9 @@ $Log: ddpro.h,v $
 #define	PDMHOST			16		/* host table stuff */
 #define	PDMSELECT		32		/* select/fd set stuff */
 #define	PDMNET			64		/* ip stuff */
+#define	PDMSCHED		256		/* scheduler stuff */
+#define	PDMAPPL			512		/* application (scrapped messages and stuff) */
+#define	PDMWAITC		1024	/* wait contexts */
 
 /* pvmd run states */
 
