@@ -1,6 +1,6 @@
 
 /*
- *           PVM 3.2:  Parallel Virtual Machine System 3.2
+ *         PVM version 3.3:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
@@ -22,8 +22,8 @@
  * purpose.  This software is provided ``as is'' without express or
  * implied warranty.
  *
- * PVM 3.2 was funded in part by the U.S. Department of Energy, the
- * National Science Foundation and the State of Tennessee.
+ * PVM version 3 was funded in part by the U.S. Department of Energy,
+ * the National Science Foundation and the State of Tennessee.
  */
 
 /*
@@ -32,6 +32,18 @@
  *	Missing links and other wonk.
  *
 $Log: pvmcruft.c,v $
+ * Revision 1.5  1994/11/07  22:48:09  manchek
+ * added tweaks for SX3, SCO
+ *
+ * Revision 1.4  1994/07/18  19:21:05  manchek
+ * pvmputenv() matched variable name substrings (need to compare = too)
+ *
+ * Revision 1.3  1994/06/03  20:38:22  manchek
+ * version 3.3.0
+ *
+ * Revision 1.2  1993/10/04  20:28:27  manchek
+ * added pvmdsockfile() function
+ *
  * Revision 1.1  1993/08/30  23:26:50  manchek
  * Initial revision
  *
@@ -57,6 +69,17 @@ $Log: pvmcruft.c,v $
 #include "global.h"
 #include "protoglarp.h"
 #include "pvmalloc.h"
+#include "tdpro.h"
+
+
+/***************
+ **  Globals  **
+ **           **
+ ***************/
+
+char *getenv();
+
+extern int pvm_useruid;				/* from pvmd.c or lpvm.c */
 
 
 /***************
@@ -64,10 +87,7 @@ $Log: pvmcruft.c,v $
  **           **
  ***************/
 
-static char rcsid[] = "$Id: pvmcruft.c,v 1.1 1993/08/30 23:26:50 manchek Exp $";
-
-
-char *getenv();
+static char rcsid[] = "$Id: pvmcruft.c,v 1.5 1994/11/07 22:48:09 manchek Exp $";
 
 
 int
@@ -143,6 +163,16 @@ getdtablesize()
 
 #else	/*IMA_TITN*/
 
+#ifdef IMA_SX3
+
+int
+getdtablesize()
+{
+	return 64*8;
+}
+
+#else /*IMA_SX3*/
+
 #include <unistd.h>
 
 int
@@ -151,11 +181,12 @@ getdtablesize()
 	return sysconf(_SC_OPEN_MAX);
 }
 
+#endif /*IMA_SX3*/
 #endif	/*IMA_TITN*/
 
 #endif /*NOGETDTBLSIZ*/
 
-#if defined(IMA_TITN) || defined(I860_NODE) || defined(I860_SRM)
+#if defined(IMA_TITN) || defined(I860_NODE) || defined(I860_SRM) || defined(IMA_SCO)
 int
 ffs(x)
 	int x;
@@ -202,7 +233,7 @@ pvmputenv(s)
 	for (p = s; *p && *p != '='; p++);
 	if (p == s || !*p)
 		return -1;
-	l = p - s;
+	l = p - s + 1;	/* must include '=' */
 
 	/* search current environ for name */
 
@@ -220,11 +251,12 @@ pvmputenv(s)
 		for (l = 2, ep = environ; *ep; l++, ep++);
 
 		if (environ == myen) {	/* mine, reallocate */
-			if (!(newen = (char**)realloc(environ, l * sizeof(char*))))
+			if (!(newen = (char**)realloc((char *)environ,
+					(unsigned)(l * sizeof(char*)))))
 				return -1;
 
 		} else {	/* not mine, alloc new and copy */
-			if (!(newen = (char**)malloc(l * sizeof(char*))))
+			if (!(newen = (char**)malloc((unsigned)(l * sizeof(char*)))))
 				return -1;
 			for (ep = environ, ep2 = newen; *ep2++ = *ep++; );
 		}
@@ -378,41 +410,17 @@ pvmgethome()
 char *
 pvmgetroot()
 {
-	char *r, *h;
 	static char *rd = 0;
 
-	if (!rd) {
-		if (!(r = getenv("PVM_ROOT")))
-			r = "pvm3";
-		if (*r == '/')
-			rd = r;
-		else {
-			h = pvmgethome();
-			rd = TALLOC(strlen(h) + strlen(r) + 2, char, "root");
-			sprintf(rd, "%s/%s", h, r);
-		}
+	if (!rd && !(rd = getenv("PVM_ROOT"))) {
+		pvmlogerror("PVM_ROOT environment variable not set.\n");
+		pvmbailout(0);
+		exit(1);		/* the other meaning of bail out */
 	}
 	return rd;
 }
 
 
-/*	pvmgetrroot()
-*
-*	Return (possibly relative to homedir) path of PVM top level directory.
-*/
-
-char *
-pvmgetrroot()
-{
-	static char *rd = 0;
-
-	if (!rd)
-		if (!(rd = getenv("PVM_ROOT")))
-			rd = "pvm3";
-	return rd;
-}
-
-
 /*	pvmgetpvmd()
 *
 *	Return absolute path of pvmd startup script.
@@ -437,27 +445,33 @@ pvmgetpvmd()
 }
 
 
-/*	pvmgetrpvmd()
+/*	pvmdsockfile()
 *
-*	Return relative (to user homedir) or absolute path of pvmd startup script.
+*	Returns name of file where pvmd advertises its sockaddr.
+*	Assumes global pvm_useruid is set before calling.
 */
 
 char *
-pvmgetrpvmd()
+pvmdsockfile()
 {
-	char *r;
-	static char *pp = 0;
+	static char buf[160];
 
-	if (!pp) {
-		if (PVMDFILE[0] == '/')
-			pp = PVMDFILE;
-		else {
-			r = pvmgetrroot();
-			pp = TALLOC(strlen(r) + strlen(PVMDFILE) + 2, char, "pvmdpath");
-			sprintf(pp, "%s/%s", r, PVMDFILE);
-		}
+	char hna[128];
+	char *p;
+
+#ifdef	SHAREDTMP
+	if (gethostname(hna, sizeof(hna)-1) == -1) {
+		pvmlogerror("pvmdsockfile() can't gethostname()\n");
+		return (char*)0;
 	}
-	return pp;
+	if (p = CINDEX(hna, '.'))
+		*p = 0;
+	(void)sprintf(buf, TDSOCKNAME, pvm_useruid, hna);
+
+#else
+	(void)sprintf(buf, TDSOCKNAME, pvm_useruid);
+#endif
+
+	return buf;
 }
 
-
