@@ -1,6 +1,6 @@
 
 /*
- *           PVM 3.2:  Parallel Virtual Machine System 3.2
+ *         PVM version 3.3:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
@@ -22,8 +22,8 @@
  * purpose.  This software is provided ``as is'' without express or
  * implied warranty.
  *
- * PVM 3.2 was funded in part by the U.S. Department of Energy, the
- * National Science Foundation and the State of Tennessee.
+ * PVM version 3 was funded in part by the U.S. Department of Energy,
+ * the National Science Foundation and the State of Tennessee.
  */
 
 /*
@@ -32,6 +32,9 @@
  *	Frag buffer util.
  *
 $Log: pvmfrag.c,v $
+ * Revision 1.2  1994/06/03  20:38:24  manchek
+ * version 3.3.0
+ *
  * Revision 1.1  1993/08/30  23:26:51  manchek
  * Initial revision
  *
@@ -52,7 +55,7 @@ extern void pvmbailout();
  **           **
  ***************/
 
-static char rcsid[] = "$Id: pvmfrag.c,v 1.1 1993/08/30 23:26:51 manchek Exp $";
+static char rcsid[] = "$Id: pvmfrag.c,v 1.2 1994/06/03 20:38:24 manchek Exp $";
 
 
 /**********************
@@ -90,7 +93,9 @@ fr_new(len)
 		fp->fr_max = 0;
 	}
 	fp->fr_len = 0;
-	fp->fr_ref = 1;
+	fp->fr_u.ref = 1;
+	fp->fr_u.dab = 1;
+	fp->fr_u.spr = 0;
 	return fp;
 
 oops:
@@ -100,6 +105,37 @@ oops:
 }
 
 
+/*	fr_snew()
+*
+*	Create a new frag with 1 reference, not in a list.
+*	Its buffer is assumed to be a writable, dense, non-databuf.
+*/
+
+struct frag *
+fr_snew(cp, len)
+	char *cp;	/* buffer */
+	int len;	/* buffer size */
+{
+	struct frag *fp;
+
+	if (!(fp = TALLOC(1, struct frag, "frag")))
+		goto oops;
+
+	fp->fr_link = fp->fr_rlink = 0;
+	fp->fr_dat = fp->fr_buf = cp;
+	fp->fr_max = fp->fr_len = len;
+	fp->fr_u.ref = 1;
+	fp->fr_u.dab = 0;
+	fp->fr_u.spr = 0;
+	return fp;
+
+oops:
+	pvmlogerror("fr_snew() can't get memory\n");
+	pvmbailout(0);
+	return (struct frag*)0;
+}
+
+
 /*	fr_unref()
 *
 *	Reduce frag refcount by 1.  If result is < 1:
@@ -112,11 +148,12 @@ void
 fr_unref(fp)
 	struct frag *fp;		/* master frag */
 {
-	if (--fp->fr_ref < 1) {
+	if (fp->fr_u.ref-- == 1) {
 		struct frag *fp2, *fp3;
 
 		if (fp->fr_buf) {		/* slave frag */
-			da_unref(fp->fr_buf);
+			if (fp->fr_u.dab)
+				da_unref(fp->fr_buf);
 
 		} else {				/* master frag */
 
