@@ -1,6 +1,6 @@
 
 /*
- *           PVM 3.2:  Parallel Virtual Machine System 3.2
+ *         PVM version 3.3:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
@@ -22,8 +22,8 @@
  * purpose.  This software is provided ``as is'' without express or
  * implied warranty.
  *
- * PVM 3.2 was funded in part by the U.S. Department of Energy, the
- * National Science Foundation and the State of Tennessee.
+ * PVM version 3 was funded in part by the U.S. Department of Energy,
+ * the National Science Foundation and the State of Tennessee.
  */
 
 /*
@@ -32,6 +32,9 @@
  *	Frag buffer util.
  *
 $Log: pvmfrag.h,v $
+ * Revision 1.2  1994/06/03  20:38:25  manchek
+ * version 3.3.0
+ *
  * Revision 1.1  1993/08/30  23:26:51  manchek
  * Initial revision
  *
@@ -40,13 +43,20 @@ $Log: pvmfrag.h,v $
 struct frag {
 	struct frag *fr_link;	/* chain or 0 */
 	struct frag *fr_rlink;
-	int fr_ref;				/* refcount (of chain if master else of frag) */
 	char *fr_buf;			/* buffer or zero if master frag */
-	int fr_max;				/* size of buffer */
 	char *fr_dat;			/* data */
+	int fr_max;				/* size of buffer */
 	int fr_len;				/* length of data */
+	struct {
+		int ref : 16;		/* refcount (of chain if master else of frag) */
+		int dab : 1;		/* buffer is a databuf */
+		int spr : 1;		/* sparse data (csz, lnc valid) */
+	} fr_u;
+	int fr_csz;				/* chunk size */
+	int fr_lnc;				/* lead to next chunk */
 };
 
-struct frag *fr_new __ProtoGlarp__((int n));
-void fr_unref __ProtoGlarp__((struct frag *f));
+struct frag *fr_new __ProtoGlarp__(( int n ));
+struct frag *fr_snew __ProtoGlarp__(( char *cp, int n ));
+void fr_unref __ProtoGlarp__(( struct frag *f ));
 
