@@ -1,6 +1,6 @@
 
 /*
- *           PVM 3.2:  Parallel Virtual Machine System 3.2
+ *         PVM version 3.3:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
@@ -22,8 +22,8 @@
  * purpose.  This software is provided ``as is'' without express or
  * implied warranty.
  *
- * PVM 3.2 was funded in part by the U.S. Department of Energy, the
- * National Science Foundation and the State of Tennessee.
+ * PVM version 3 was funded in part by the U.S. Department of Energy,
+ * the National Science Foundation and the State of Tennessee.
  */
 
 /*
@@ -32,6 +32,12 @@
  *	Libpvm message descriptors
  *
 $Log: pvmumbuf.c,v $
+ * Revision 1.3  1994/06/04  21:46:24  manchek
+ * ripped out old cruft
+ *
+ * Revision 1.2  1994/06/03  20:38:26  manchek
+ * version 3.3.0
+ *
  * Revision 1.1  1993/08/30  23:26:51  manchek
  * Initial revision
  *
@@ -62,7 +68,7 @@ int pvmmidhsiz = 0;					/* length of pvmmidh */
  **           **
  ***************/
 
-static char rcsid[] = "$Id: pvmumbuf.c,v 1.1 1993/08/30 23:26:51 manchek Exp $";
+static char rcsid[] = "$Id: pvmumbuf.c,v 1.3 1994/06/04 21:46:24 manchek Exp $";
 static char pvmtxt[512];			/* scratch for error log */
 static int pvmmidhfree = 0;			/* head of midh free list */
 
@@ -204,8 +210,9 @@ midtobuf(mid)
 }
 
 
-umbuf_dump(mid)
+umbuf_dump(mid, lvl)
 	int mid;
+	int lvl;	/* 0 - summary, 1 - frag lengths, 2 - frag data */
 {
 	struct umbuf *up;
 	struct frag *fp;
@@ -222,23 +229,26 @@ umbuf_dump(mid)
 	}
 
 	up = pvmmidh[mid].m_umb;
-	sprintf(pvmtxt, "umbuf_dump() mid=%d 0x%x len=%d cod=%d enc=%d src=t%x\n",
-			mid, up, up->ub_len, up->ub_cod, up->ub_enc, up->ub_src);
+	if (up->ub_flag & UB_PACK)
+		setublen(up);
+	sprintf(pvmtxt, "umbuf_dump() mid=%d 0x%x src=t%x enc=%d cod=%d len=%d\n",
+			mid, up, up->ub_src, up->ub_enc, up->ub_cod, up->ub_len);
 	pvmlogerror(pvmtxt);
-	for (fp = up->ub_frag->fr_link; fp != up->ub_frag; fp = fp->fr_link) {
-		sprintf(pvmtxt, " frag=0x%x max=%d ofs=%d len=%d\n",
-			fp, fp->fr_max, fp->fr_dat - fp->fr_buf, fp->fr_len);
-		pvmlogerror(pvmtxt);
-		pvmhdump(fp->fr_dat, fp->fr_len, "  ");
-/*
-		pvmhdump(fp->fr_dat, 32, "  ");
-*/
+	if (lvl > 0) {
+		for (fp = up->ub_frag->fr_link; fp != up->ub_frag; fp = fp->fr_link) {
+			sprintf(pvmtxt, " frag=0x%x max=%d ofs=%d len=%d\n",
+				fp, fp->fr_max, fp->fr_dat - fp->fr_buf, fp->fr_len);
+			pvmlogerror(pvmtxt);
+			if (lvl > 1)
+				pvmhdump(fp->fr_dat, fp->fr_len, "  ");
+		}
 	}
 	return 0;
 }
 
 
-umbuf_list()
+umbuf_list(lvl)
+	int lvl;
 {
 	int i;
 	struct umbuf *up;
@@ -246,16 +256,22 @@ umbuf_list()
 	int rlen;
 
 	for (i = 1; i < pvmmidhsiz; i++)
-		if (up = pvmmidh[i].m_umb) {
-			rlen = 0;
-			for (fp = up->ub_frag->fr_link; fp != up->ub_frag;
-					fp = fp->fr_link)
-				rlen += fp->fr_max;
-			sprintf(pvmtxt,
-			"umbuf_list() mid=%d cod=%d enc=%d src=t%x len=%d rlen=%d\n",
-					i, up->ub_cod, up->ub_enc, up->ub_src, up->ub_len, rlen);
-			pvmlogerror(pvmtxt);
-		}
+		if (up = pvmmidh[i].m_umb)
+			umbuf_dump(i, lvl);
+}
+
+
+int
+setublen(up)
+	struct umbuf *up;
+{
+	struct frag *fp, *fp2;
+	int l = 0;
+
+	fp = fp2 = up->ub_frag;
+	while ((fp = fp->fr_link) != fp2)
+		l += fp->fr_len;
+	return up->ub_len = l;
 }
 
 
