@@ -1,6 +1,6 @@
 
 /*
- *           PVM 3.2:  Parallel Virtual Machine System 3.2
+ *         PVM version 3.3:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
@@ -22,8 +22,8 @@
  * purpose.  This software is provided ``as is'' without express or
  * implied warranty.
  *
- * PVM 3.2 was funded in part by the U.S. Department of Energy, the
- * National Science Foundation and the State of Tennessee.
+ * PVM version 3 was funded in part by the U.S. Department of Energy,
+ * the National Science Foundation and the State of Tennessee.
  */
 
 /*
@@ -32,21 +32,57 @@
  *	Exec more pvmds.  It's good for you.
  *
 $Log: startup.c,v $
+ * Revision 1.9  1994/10/15  19:31:31  manchek
+ * fixed typo in log message
+ *
+ * Revision 1.8  1994/06/04  21:45:24  manchek
+ * added unix domain sockets.
+ * ripped out old (serial) startup code
+ *
+ * Revision 1.7  1994/06/03  20:38:27  manchek
+ * version 3.3.0
+ *
+ * Revision 1.6  1993/11/30  19:54:41  manchek
+ * check the default entry in filehosts when adding new hosts
+ *
+ * Revision 1.5  1993/11/30  16:46:10  manchek
+ * pass whole remote command as a single arg to rsh
+ *
+ * Revision 1.4  1993/11/30  15:54:37  manchek
+ * master pvmd once again doesn't close fds 0..2 -
+ * this broke rexec startup
+ *
+ * Revision 1.3  1993/10/25  20:53:51  manchek
+ * fixed a few typos in error log messages.
+ * added code to close all fds and reopen 0..2 as /dev/null
+ *
+ * Revision 1.2  1993/10/04  20:30:30  manchek
+ * mksocks() now uses pvmdsockfile() instead of TDSOCKNAME
+ *
  * Revision 1.1  1993/08/30  23:26:51  manchek
  * Initial revision
  *
  */
 
+#ifdef IMA_BSD386
+#include <machine/endian.h>
+#endif
+#ifdef IMA_LINUX
+#include <endian.h>
+#endif
 #ifdef IMA_TITN
 #include <bsd/sys/types.h>
 #else
 #include <sys/types.h>
 #endif
 #include <sys/time.h>
-#ifdef IMA_RS6K
+#if defined(IMA_RS6K) || defined(IMA_POWER4)
 #include <sys/select.h>
 #endif
 #include <sys/socket.h>
+#ifndef NOUNIXDOM
+#include <sys/un.h>
+#endif
 #include <netinet/in.h>
 #include <netinet/tcp.h>
 #include <fcntl.h>
@@ -66,7 +102,6 @@ $Log: startup.c,v $
 #include "global.h"
 #include "fromlib.h"
 #include "ddpro.h"
-#include "tdpro.h"
 #include "protoglarp.h"
 #include "pvmalloc.h"
 #include "mesg.h"
@@ -102,16 +137,46 @@ $Log: startup.c,v $
 #endif
 
 
+struct hst {
+	int h_tid;
+	char *h_name;
+	char *h_login;
+	char *h_sopts;
+	int h_flag;
+#define	HST_PASSWORD	1		/* ask for a password */
+#define	HST_MANUAL		2		/* do manual startup */
+	char *h_cmd;
+	char *h_result;
+};
+
+#ifdef	NOTMPNAM
+#define	TMPNAMFUN(x)	pvmtmpnam(x)
+#define	LEN_OF_TMP_NAM	32
+char *pvmtmpnam();
+
+#else	/*NOTMPNAM*/
+#define	TMPNAMFUN(x)	tmpnam(x)
+#ifdef	L_tmpnam
+#define	LEN_OF_TMP_NAM	L_tmpnam
+#else
+#define	LEN_OF_TMP_NAM	64
+#endif
+#endif	/*NOTMPNAM*/
+
+
 /***************
  **  Globals  **
  **           **
  ***************/
 
+char *getenv();
+
 extern int errno;
 
 extern char *inadport_hex();
 extern void pvmbailout();
-extern char *pvmgetrpvmd();
+char *pvmdsockfile();
+char *pvmgethome();
 
 extern int debugmask;				/* from pvmd.c */
 extern char *debugger;				/* from pvmd.c */
@@ -120,6 +185,9 @@ extern struct htab *filehosts;		/* from pvmd.c */
 extern struct htab *hosts;			/* from pvmd.c */
 extern char *loclsnam;				/* from pvmd.c */
 extern int loclsock;				/* from pvmd.c */
+#ifndef NOUNIXDOM
+extern char *loclspath;				/* from pvmd.c */
+#endif
 extern int log_fd;					/* from logging.c */
 extern int log_how;					/* from logging.c */
 extern char *myarchname;			/* from pvmd.c */
@@ -129,10 +197,8 @@ extern int netsock;					/* from pvmd.c */
 extern struct htab *oldhosts;		/* from pvmd.c */
 extern int ourudpmtu;				/* from pvmd.c */
 extern int ppnetsock;				/* from pvmd.c */
-extern int pprwid;					/* from pvmd.c */
 extern int runstate;				/* from pvmd.c */
 extern int tidhmask;				/* from pvmd.c */
-extern int useruid;					/* from pvmd.c */
 extern char *username;				/* from pvmd.c */
 
 
@@ -141,7 +207,7 @@ extern char *username;				/* from pvmd.c */
  **           **
  ***************/
 
-static char rcsid[] = "$Id: startup.c,v 1.1 1993/08/30 23:26:51 manchek Exp $";
+static char rcsid[] = "$Id: startup.c,v 1.9 1994/10/15 19:31:31 manchek Exp $";
 static char pvmtxt[1024];		/* scratch for error log */
 
 
@@ -150,7 +216,9 @@ static char pvmtxt[1024];		/* scratch for error log */
 *	Make UDP sockets netsock and ppnetsock.  Make TCP master socket
 *	loclsock.
 *
-*	Returns 0 if ok else 1.
+*	Returns 0 if ok,
+*	else 2 if pvmd already running,
+*	else 1.
 */
 
 int
@@ -160,12 +228,17 @@ mksocs()
 	struct hostd *hp0 = hosts->ht_hosts[0];
 	struct sockaddr_in sin;
 	char buf[128];
+	char *sfn;
 	int d;
 #ifndef NOSOCKOPT
 	int bsz;
 #endif
 	char *p;
 	int cc;
+#ifndef NOUNIXDOM
+	char spath[LEN_OF_TMP_NAM];	/* local socket path */
+	struct sockaddr_un uns;
+#endif
 
 	/*
 	* make pvmd-pvmd socket
@@ -213,6 +286,7 @@ mksocs()
 	* make pvmd-local task socket
 	*/
 
+#ifdef NOUNIXDOM
 	if ((loclsock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
 		pvmlogperror("mksocs() socket loclsock");
 		return 1;
@@ -246,54 +320,6 @@ mksocs()
 		return 1;
 	}
 
-	/*
-	* make pvmd-local task socket address file
-	*/
-
-	p = inadport_hex(&sin);
-	(void)sprintf(buf, TDSOCKNAME, useruid);
-	if ((d = open(buf, O_CREAT|O_EXCL|O_WRONLY|O_TRUNC, 0600)) == -1) {
-		if (errno == EEXIST) {
-#ifndef	OVERLOADHOST
-			(void)sprintf(pvmtxt,
-					"mksocs() %s exists.  pvmd already running?\n", buf);
-			pvmlogerror(pvmtxt);
-			return 1;
-#endif
-
-		} else {
-			pvmlogperror(buf);
-			pvmlogerror("mksocs() can't write address file\n");
-			return 1;
-		}
-
-	} else {
-		cc = write(d, p, strlen(p));
-		if (cc != strlen(p)) {
-			if (cc == -1) {
-				pvmlogperror(buf);
-				pvmlogerror("mksocs() can't write address file\n");
-
-			} else {
-				(void)sprintf(pvmtxt,
-						"mksocs() aargh, short write on %s: %d\n", buf, cc);
-				pvmlogerror(pvmtxt);
-				pvmlogerror("mksocs() is the partition full?\n");
-			}
-			(void)close(d);
-			(void)unlink(buf);
-			return 1;
-		}
-		loclsnam = STRALLOC(buf);
-		(void)close(d);
-	}
-
-	/* set PVMSOCK envar */
-
-	sprintf(buf, "PVMSOCK=%s", p);
-	p = STRALLOC(buf);
-	pvmputenv(p);
-
 #ifndef NOSOCKOPT
 	bsz = ourudpmtu * 2;
 	if (setsockopt(netsock, SOL_SOCKET, SO_SNDBUF,
@@ -308,52 +334,189 @@ mksocs()
 		return 1;
 	}
 #endif /*NOSOCKOPT*/
+
+	p = inadport_hex(&sin);
+
+#else /*NOUNIXDOM*/
+	if ((loclsock = socket(AF_UNIX, SOCK_STREAM, 0)) == -1) {
+		pvmlogperror("mksocs() socket loclsock");
+		return 1;
+	}
+
+	BZERO((char*)&uns, sizeof(uns));
+	uns.sun_family = AF_UNIX;
+	spath[0] = 0;
+	(void)TMPNAMFUN(spath);
+	strcpy(uns.sun_path, spath);
+/*
+XXX len?
+*/
+
+	if (bind(loclsock, (struct sockaddr*)&uns, sizeof(uns)) == -1) {
+		pvmlogperror("mksocs() bind loclsock");
+		return 1;
+	}
+
+	if (listen(loclsock, SOMAXCONN) == -1) {
+		pvmlogperror("mksocs() listen loclsock");
+		return 1;
+	}
+
+	loclspath = STRALLOC(spath);
+	p = spath;
+
+#endif /*NOUNIXDOM*/
+
+	/*
+	* make pvmd-local task socket address file
+	*/
+
+	if (!(sfn = pvmdsockfile())) {
+		pvmlogerror("mksocs() pvmdsockfile() failed\n");
+		pvmbailout(0);
+	}
+
+	if ((d = open(sfn, O_CREAT|O_EXCL|O_WRONLY|O_TRUNC, 0600)) == -1) {
+		if (errno == EEXIST) {
+#ifndef	OVERLOADHOST
+			(void)sprintf(pvmtxt,
+					"mksocs() %s exists.  pvmd already running?\n", sfn);
+			pvmlogerror(pvmtxt);
+			return 2;
+#endif
+
+		} else {
+			pvmlogperror(sfn);
+			pvmlogerror("mksocs() can't write address file\n");
+			return 1;
+		}
+
+	} else {
+		cc = write(d, p, strlen(p));
+		if (cc != strlen(p)) {
+			if (cc == -1) {
+				pvmlogperror(sfn);
+				pvmlogerror("mksocs() can't write address file\n");
+
+			} else {
+				(void)sprintf(pvmtxt,
+						"mksocs() aargh, short write on %s: %d\n", sfn, cc);
+				pvmlogerror(pvmtxt);
+				pvmlogerror("mksocs() is the partition full?\n");
+			}
+			(void)close(d);
+			(void)unlink(sfn);
+			return 1;
+		}
+		loclsnam = STRALLOC(sfn);
+		(void)close(d);
+	}
+
+	/* set PVMSOCK envar */
+
+	sprintf(buf, "PVMSOCK=%s", p);
+	p = STRALLOC(buf);
+	pvmputenv(p);
+
 	return 0;
 }
 
 
-/*	pathsep()
+/*	colonsep()
 *
-*	Break path string into one or more strings on a delimiter character.
-*	Also do macro substitution on another character.
+*	Break string into substrings on ':' delimiter.
 *	Return null-terminated array of strings, in new malloc'd space.
+*	Modifies the original string.
 */
 
 char **
-pathsep(s, bc, ms, mc)
+colonsep(s)
 	char *s;	/* the string to break up */
-	char bc;	/* the char to break on */
-	char *ms;	/* string to replace macro char */
-	char mc;	/* macro char */
 {
 	char **els;
 	int nel = 2;			/* length of els */
-	char *p, *q, *r;
-	char buf[1024];			/* XXX danger, static limit */
+	char *p, *q;
 
-	for (p = s; p = CINDEX(p, bc); p++)
+	for (p = s; p = CINDEX(p, ':'); p++)
 		nel++;
-	els = TALLOC(nel, char*, "psep");
+	els = TALLOC(nel, char*, "path");
 
 	nel = 0;
 	for (p = s; p; p = q) {
-		if (q = CINDEX(p, bc))
+		if (q = CINDEX(p, ':'))
 			*q++ = 0;
-		for (r = buf; *p; p++) {
-			if (*p == mc) {
-				strcpy(r, ms);
-				r += strlen(r);
-			} else
-				*r++ = *p;
-		}
-		*r = 0;
-		els[nel++] = STRALLOC(buf);
+		els[nel++] = p;
 	}
 	els[nel] = 0;
 	return els;
 }
 
 
+/*	varsub()
+*
+*	Substitute environment variables into string.
+*	Variables named by $NAME or ${NAME}.
+*	Return string in new malloc'd space.
+*/
+
+char *
+varsub(s)
+	char *s;
+{
+	int rm = 8;		/* length of result string space */
+	char *r;		/* result string */
+	int rl = 0;
+	char *p;
+	char *vn, *vv;
+	char c;
+	int l;
+
+	r = TALLOC(rm, char, "var");
+	while (*s) {
+		for (p = s; *p && *p != '$'; p++) ;
+		if (l = p - s) {
+			if (rl + l >= rm) {
+				rm = rl + l + 1;
+				r = TREALLOC(r, rm, char);
+			}
+			strncpy(r + rl, s, l);
+			rl += l;
+		}
+		s = p++;
+		if (*s == '$') {
+			if (*p == '{')
+				p++;
+			vn = p;
+			while (isalnum(*p) || *p == '_')
+				p++;
+			c = *p;
+			*p = 0;
+			vv = getenv(vn);
+			*p = c;
+			if (*p == '}')
+				p++;
+			if (vv)
+				l = strlen(vv);
+			else {
+				vv = s;
+				l = p - s;
+			}
+			if (l) {
+				if (rl + l >= rm) {
+					rm = rl + l + 1;
+					r = TREALLOC(r, rm, char);
+				}
+				strncpy(r + rl, vv, l);
+				rl += l;
+			}
+			s = p;
+		}
+	}
+	r[rl] = 0;
+	return r;
+}
+
+
 /*	crunchzap()
 *
 *	Parse a string into words delimited by <> pairs.
@@ -407,13 +570,24 @@ master_config(hn, argc, argv)
 	struct hostent *he;
 	struct hostd *hp;
 	struct hostd *hp2;
+	int i;
+	char *s;
 
 	if (argc > 2) {
 		pvmlogerror("usage: pvmd3 [-ddebugmask] [-nhostname] [hostfile]\n");
 		pvmbailout(0);
 	}
-	if (argc == 2)
+	if (argc == 2) {
 		filehosts = readhostfile(argv[1]);
+	}
+	if (debugmask & PDMSTARTUP) {
+		if (filehosts) {
+			pvmlogerror("master_config() host file:\n");
+			ht_dump(filehosts);
+
+		} else
+			pvmlogerror("master_config() null host file\n");
+	}
 
 	hosts = ht_new(1);
 	hosts->ht_serial = 1;
@@ -421,18 +595,10 @@ master_config(hn, argc, argv)
 	hosts->ht_cons = 1;
 	hosts->ht_local = 1;
 
-	if (!(he = gethostbyname(hn))) {
-		sprintf(pvmtxt, "main() %s: can't gethostbyname\n", hn);
-		pvmlogerror(pvmtxt);
-		pvmbailout(0);
-	}
-
 	hp = hd_new(1);
 	hp->hd_name = STRALLOC(hn);
 	hp->hd_arch = STRALLOC(myarchname);
 	hp->hd_mtu = UDPMAXLEN;
-	BCOPY(he->h_addr_list[0], (char*)&hp->hd_sad.sin_addr,
-		sizeof(struct in_addr));
 	ht_insert(hosts, hp);
 	hd_unref(hp);
 
@@ -440,31 +606,68 @@ master_config(hn, argc, argv)
 	hp->hd_name = STRALLOC("pvmd'");
 	hp->hd_arch = STRALLOC(myarchname);
 	hp->hd_mtu = UDPMAXLEN;
-	BCOPY(he->h_addr_list[0], (char*)&hp->hd_sad.sin_addr,
-		sizeof(struct in_addr));
 	ht_insert(hosts, hp);
 	hd_unref(hp);
 
-	if (mksocs())
-		pvmbailout(0);
-
 	/*
 	* get attributes from host file if available
 	*/
 
 	hp = hosts->ht_hosts[1];
-	if (filehosts && (hp2 = nametohost(filehosts, hp->hd_name))) {
-		if (hp2->hd_epath)
-			hp->hd_epath = STRALLOC(hp2->hd_epath);
-		if (hp2->hd_bpath)
-			hp->hd_bpath = STRALLOC(hp2->hd_bpath);
-		hp->hd_speed = hp2->hd_speed;
+	if (filehosts &&
+			((hp2 = nametohost(filehosts, hp->hd_name))
+			|| (hp2 = filehosts->ht_hosts[0]))) {
+		applydefaults(hp, hp2);
 	}
 
 	if (!hp->hd_epath)
 		hp->hd_epath = STRALLOC(DEFBINDIR);
-	epaths = pathsep(hp->hd_epath, ':', myarchname, '%');
-	debugger = hp->hd_bpath ? hp->hd_bpath : STRALLOC(DEFDEBUGGER);
+	epaths = colonsep(varsub(hp->hd_epath));
+	if (!hp->hd_bpath)
+		hp->hd_bpath = STRALLOC(DEFDEBUGGER);
+	debugger = varsub(hp->hd_bpath);
+	if (!hp->hd_wdir)
+		hp->hd_wdir = STRALLOC(pvmgethome());
+	s = varsub(hp->hd_wdir);
+	if (chdir(s) == -1)
+		pvmlogperror(s);
+	PVM_FREE(s);
+
+	if (!(he = gethostbyname(hp->hd_aname ? hp->hd_aname : hp->hd_name))) {
+		sprintf(pvmtxt, "master_config() %s: can't gethostbyname\n", hn);
+		pvmlogerror(pvmtxt);
+		pvmbailout(0);
+	}
+
+	BCOPY(he->h_addr_list[0], (char*)&hp->hd_sad.sin_addr,
+		sizeof(struct in_addr));
+
+	hp = hosts->ht_hosts[0];
+	BCOPY(he->h_addr_list[0], (char*)&hp->hd_sad.sin_addr,
+		sizeof(struct in_addr));
+
+	if (mksocs())
+		pvmbailout(0);
+
+	/* print local socket address on stdout in case someone cares */
+
+	printf("%s\n", getenv("PVMSOCK"));
+	fflush(stdout);
+
+	/* close everything but our sockets */
+
+	for (i = getdtablesize(); --i > 2; )
+/* XXX don't like this - hard to maintain */
+		if (i != netsock && i != ppnetsock && i != loclsock && i != log_fd)
+			(void)close(i);
+
+	/* reopen 0, 1, 2*/
+
+	(void)open("/dev/null", O_RDONLY, 0);
+	(void)open("/dev/null", O_WRONLY, 0);
+	(void)dup2(1, 2);
+
+	log_how &= ~1;
 
 	runstate = PVMDNORMAL;
 	return 0;
@@ -485,16 +688,39 @@ slave_config(hn, argc, argv)
 	int lh;			/* local host index */
 	int mh;			/* master host index */
 	struct hostd *hp;
-	int i;
+	int i, j;
+	int ac;
 	int ms = 0;		/* manual (humanoid) startup */
+	int dof = 1;	/* fork, exit parent (default) */
+	int bad = 0;
+	char *p;
 
-	if (argc == 7 && !strcmp(argv[1], "-S")) {
-		ms = 1;
-		argv++;
-		argc--;
+	for (i = j = ac = 1; i < argc; i++) {
+		if (argv[i][0] == '-') {
+			switch (argv[i][1]) {
+
+			case 'S':
+				ms = 1;
+				break;
+
+			case 'f':
+				dof = 0;
+				break;
+
+			default:
+				sprintf(pvmtxt, "slave_config() unknown switch: %s\n", argv[i]);
+				pvmlogerror(pvmtxt);
+				bad++;
+			}
+
+		} else {
+			argv[j++] = argv[i];
+			ac++;
+		}
 	}
+	argc = ac;
 
-	if (argc != 6) {
+	if (bad || argc != 6) {
 		pvmlogerror("slave_config: bad args\n");
 		pvmbailout(0);
 	}
@@ -530,8 +756,13 @@ slave_config(hn, argc, argv)
 	ht_insert(hosts, hp);
 	hd_unref(hp);
 
-	if (mksocs())
+	if (i = mksocs()) {
+		if (i == 2) {
+			printf("PvmDupHost\n");
+			fflush(stdout);
+		}
 		pvmbailout(0);
+	}
 
 	printf("ddpro<%d> arch<%s> ip<%s> mtu<%d>\n",
 		DDPROTOCOL,
@@ -541,183 +772,129 @@ slave_config(hn, argc, argv)
 	fflush(stdout);
 
 	if (!ms)
-		read(0, (char*)&i, 1);
+		(void)read(0, (char*)&i, 1);
 
-	if (i = fork()) {
-		if (i == -1)
-			pvmlogperror("slave_config() fork");
-		exit(0);
+	if (dof) {
+		if (i = fork()) {
+			if (i == -1)
+				pvmlogperror("slave_config() fork");
+			exit(0);
+		}
+
+	/* close everything but our sockets */
+
+		for (i = getdtablesize(); --i >= 0; )
+/* XXX don't like this - hard to maintain */
+			if (i != netsock && i != loclsock && i != log_fd)
+				(void)close(i);
 	}
 
-	for (i = getdtablesize(); --i >= 0; )
-/* XXX don't like this - hard to maintain */
-		if (i != netsock && i != loclsock && i != log_fd)
-			(void)close(i);
+	/* reopen 0, 1, 2*/
+
+	(void)open("/dev/null", O_RDONLY, 0);
+	(void)open("/dev/null", O_WRONLY, 0);
+	(void)dup2(1, 2);
 
 	log_how &= ~1;
 
-	if ((myunixpid = getpid()) == -1) {
-		pvmlogerror("main() can't getpid()\n");
-		pvmbailout(0);
-	}
-
-	epaths = pathsep(STRALLOC(DEFBINDIR), ':', myarchname, '%');
-	debugger = STRALLOC(DEFDEBUGGER);
+	p = STRALLOC(DEFBINDIR);
+	epaths = colonsep(varsub(p));
+	PVM_FREE(p);
+	p = STRALLOC(DEFDEBUGGER);
+	debugger = varsub(p);
+	PVM_FREE(p);
 	runstate = PVMDSTARTUP;
 	return 0;
 }
 
 
-/*	start_slaves()
-*
-*	This is the main function for pvmd'.  It attempts to start slave
-*	pvmds on a list of hosts.
-*/
-
-start_slaves(htp)
-	struct htab *htp;		/* table of new slaves */
+int
+hoster(mp)
+	struct mesg *mp;
 {
-	struct mesg *mp;		/* to inform parent pvmd when finished */
-	struct hostent *he;
-	struct hostd *hp, *hp2;
-	int hh, hh2;
+	struct mesg *mp2;
+	int num;
+	int i;
+	struct hst **hostlist;
+	struct hst *hp;
+	char *p;
 
 	/*
-	* for each host in table without errors, get ipaddr
+	* unpack the startup message
 	*/
 
-	for (hh = 1; hh <= htp->ht_last; hh++) {
-		hp = htp->ht_hosts[hh];
-		if (!hp || hp->hd_err)
-			continue;
-
-	/*
-	* first check table from host file if available
-	*/
-
-		if (filehosts && (hp2 = nametohost(filehosts, hp->hd_name))) {
-			hp->hd_flag |= hp2->hd_flag;
-			if (hp2->hd_login)
-				hp->hd_login = STRALLOC(hp2->hd_login);
-			if (hp2->hd_dpath)
-				hp->hd_dpath = STRALLOC(hp2->hd_dpath);
-			if (hp2->hd_epath)
-				hp->hd_epath = STRALLOC(hp2->hd_epath);
-			if (hp2->hd_bpath)
-				hp->hd_bpath = STRALLOC(hp2->hd_bpath);
-			BCOPY((char*)&hp2->hd_sad.sin_addr, (char*)&hp->hd_sad.sin_addr,
-					sizeof(struct in_addr));
-			hp->hd_speed = hp2->hd_speed;
-
-		} else {
-			if (he = gethostbyname(hp->hd_name)) {
-				BCOPY(he->h_addr_list[0], (char*)&hp->hd_sad.sin_addr,
-						sizeof(struct in_addr));
+	upkint(mp, &num);
+	if (debugmask & PDMSTARTUP) {
+		sprintf(pvmtxt, "hoster() %d to start\n", num);
+		pvmlogerror(pvmtxt);
+	}
+	if (num > 0) {
+		hostlist = TALLOC(num, struct hst *, "xxx");
+		for (i = 0; i < num; i++) {
+			hp = TALLOC(1, struct hst, "xxx");
+			hostlist[i] = hp;
+			hp->h_flag = 0;
+			hp->h_result = 0;
+			if (upkint(mp, &hp->h_tid)
+			|| upkstralloc(mp, &hp->h_sopts)
+			|| upkstralloc(mp, &hp->h_login)
+			|| upkstralloc(mp, &hp->h_cmd)) {
+				pvmlogerror("hoster() bad message format\n");
+				pvmbailout(0);
+			}
+			if (debugmask & PDMSTARTUP) {
+				sprintf(pvmtxt, "%d. t%x %s so=\"%s\"\n", i,
+						hp->h_tid,
+						hp->h_login,
+						hp->h_sopts);
+				pvmlogerror(pvmtxt);
+			}
+			if (p = CINDEX(hp->h_login, '@')) {
+				hp->h_name = STRALLOC(p + 1);
+				*p = 0;
+				p = STRALLOC(hp->h_login);
+				PVM_FREE(hp->h_login);
+				hp->h_login = p;
 
 			} else {
-				if (debugmask & PDMSTARTUP) {
-					sprintf(pvmtxt, "start_slaves() can't gethostbyname: %s\n",
-							hp->hd_name);
-					pvmlogerror(pvmtxt);
-				}
-				hp->hd_err = PvmNoHost;
-			}
-		}
-
-	/*
-	* check that it's not already configured
-	*/
-
-		if (!hp->hd_err) {
-			for (hh2 = oldhosts->ht_last; hh2 > 0; hh2--) {
-				if ((hp2 = oldhosts->ht_hosts[hh2])
-				&& (hp2->hd_sad.sin_addr.s_addr == hp->hd_sad.sin_addr.s_addr))
-					hp->hd_err = PvmDupHost;
+				hp->h_name = hp->h_login;
+				hp->h_login = 0;
 			}
+			if (!strcmp(hp->h_sopts, "pw"))
+				hp->h_flag |= HST_PASSWORD;
+			if (!strcmp(hp->h_sopts, "ms"))
+				hp->h_flag |= HST_MANUAL;
 		}
 	}
 
 	/*
-	* check that new hosts aren't duplicated
+	* do it
 	*/
 
-	for (hh = htp->ht_last; hh > 0; hh--) {
-		if (!(hp = htp->ht_hosts[hh]) || hp->hd_err)
-			continue;
-		for (hh2 = hh; --hh2 > 0; )
-			if ((hp2 = htp->ht_hosts[hh2])
-			&& !hp2->hd_err
-			&& (hp2->hd_sad.sin_addr.s_addr == hp->hd_sad.sin_addr.s_addr))
-				hp->hd_err = PvmDupHost;
-	}
+	pl_startup(num, hostlist);
 
 	/*
-	* try to start each host that doesn't have an error
+	* send results back to pvmd
 	*/
 
-
-	if (debugmask & PDMSTARTUP) {
-		pvmlogerror("start_slaves() using host table:\n");
-		ht_dump(htp);
-	}
-
-#if	RSHNPLL > 1
-	pl_startup(htp);
-
-#else	/*RSHNPLL > 1*/
-	for (hh = 1; hh <= htp->ht_last; hh++) {
-		hp = htp->ht_hosts[hh];
-		if (!hp || hp->hd_err)
-			continue;
-		slave_exec(hp);
-	}
-#endif	/*RSHNPLL > 1*/
-
-	if (debugmask & PDMSTARTUP) {
-		pvmlogerror("start_slaves() result host table:\n");
-		ht_dump(htp);
-	}
-
-	/*
-	* send host table with addresses and errors back to parent
-	*/
-
-	mp = mesg_new(0);
-	mp->m_dst = hosts->ht_hosts[hosts->ht_master]->hd_hostpart | TIDPVMD;
-	mp->m_cod = DM_STARTACK;
-	mp->m_wid = pprwid;
-	pkint(mp, htp->ht_cnt);
-	for (hh = 1; hh <= htp->ht_last; hh++) {
-		if (hp = htp->ht_hosts[hh]) {
-			pkint(mp, hh);
-			pkint(mp, hp->hd_err);
-			if (!hp->hd_err) {
-				pkstr(mp, hp->hd_arch);
-				pkstr(mp, inadport_hex(&hp->hd_sad));
-				pkint(mp, hp->hd_mtu);
-				if (hp->hd_epath) {
-					pkint(mp, DM_SLCONF_EP);
-					pkstr(mp, hp->hd_epath);
-				}
-				if (hp->hd_bpath) {
-					pkint(mp, DM_SLCONF_BP);
-					pkstr(mp, hp->hd_bpath);
-				}
-				pkint(mp, 0);
-			}
-		}
+	mp2 = mesg_new(0);
+	mp2->m_dst = mp->m_src;
+	mp2->m_cod = DM_STARTACK;
+	mp2->m_wid = mp->m_wid;
+	pkint(mp2, num);
+	for (i = 0; i < num; i++) {
+		pkint(mp2, hostlist[i]->h_tid);
+		pkstr(mp2, hostlist[i]->h_result
+				? hostlist[i]->h_result : "PvmDSysErr");
 	}
 	if (debugmask & PDMSTARTUP)
-		pvmlogerror("start_slaves() pvmd' sending back host table\n");
-	sendmessage(mp);
-
-	work();
+		pvmlogerror("hoster() pvmd' sending back host table\n");
+	sendmessage(mp2);
+	work();		/* no return */
 	return 0;	/* not reached */
 }
 
 
-#if RSHNPLL > 1
-
 /********************************************
 *  this is the new (parallel) startup code  *
 *                                           *
@@ -725,7 +902,7 @@ start_slaves(htp)
 
 struct slot {
 	struct slot *s_link, *s_rlink;		/* free/active list */
-	struct hostd *s_hostd;				/* host table entry */
+	struct hst *s_hst;					/* host table entry */
 	struct timeval s_bail;				/* timeout time */
 	int s_rfd, s_wfd, s_efd;			/* slave stdin/out/err */
 	char s_buf[256];					/* config reply line */
@@ -751,23 +928,22 @@ close_slot(sp)
 }
 
 
-pl_startup(htp)
-	struct htab *htp;
+pl_startup(num, hostlist)
+	int num;
+	struct hst **hostlist;
 {
-	int hhnext = 1;						/* next host in htp to try */
+	int nxth = 0;						/* next host in list to start */
 	struct slot *slact = 0;				/* active list of slots */
-	struct hostd *hp;
+	struct hst *hp;
 	struct slot *sp, *sp2;
-	int i;
 	struct timeval tnow;
 	struct timeval tout;
-	int n;
 	struct fd_set rfds;
 	int nfds;
+	int i;
+	int n;
+	char *p;
 	char ebuf[256];						/* for reading stderr */
-	int ver;
-	char *av[16];						/* for reply parsing */
-	int ac;
 
 	/* init slot free list */
 
@@ -792,28 +968,22 @@ pl_startup(htp)
 		for (; ; ) {
 
 			/* find a host for slot */
-			hp = 0;
-			if (slfree->s_link != slfree)
-				while (hhnext <= htp->ht_last)
-					if (hp = htp->ht_hosts[hhnext++]) {
-						if (hp->hd_err)
-							hp = 0;
-						else
-							break;
-					}
-			if (!hp)
+
+			if (slfree->s_link != slfree && nxth < num)
+				hp = hostlist[nxth++];
+			else
 				break;
 
 			sp = slfree->s_link;
 			LISTDELETE(sp, s_link, s_rlink);
-			sp->s_hostd = hp;
+			sp->s_hst = hp;
 			sp->s_len = 0;
 			if (debugmask & PDMSTARTUP) {
-				sprintf(pvmtxt, "pl_startup() trying %s\n", hp->hd_name);
+				sprintf(pvmtxt, "pl_startup() trying %s\n", hp->h_name);
 				pvmlogerror(pvmtxt);
 			}
 			phase1(sp);
-			if (hp->hd_err || hp->hd_arch) {
+			if (hp->h_result) {
 				/* error or fully started (manual startup) */
 
 				LISTPUTBEFORE(slfree, sp, s_link, s_rlink);
@@ -845,9 +1015,9 @@ pl_startup(htp)
 		for (sp = slact->s_link; sp != slact; sp = sp->s_link) {
 			if (TVXLTY(&sp->s_bail, &tnow)) {
 				sprintf(pvmtxt, "pl_startup() %s timed out after %d secs\n",
-						sp->s_hostd->hd_name, RSHTIMEOUT);
+						sp->s_hst->h_name, RSHTIMEOUT);
 				pvmlogerror(pvmtxt);
-				sp->s_hostd->hd_err = PvmCantStart;
+				sp->s_hst->h_result = STRALLOC("PvmCantStart");
 				sp2 = sp->s_rlink;
 				close_slot(sp);
 				sp = sp2;
@@ -883,7 +1053,7 @@ pl_startup(htp)
 		}
 		if ((n = select(nfds, &rfds, (fd_set*)0, (fd_set*)0, &tout)) == -1) {
 			if (errno != EINTR) {
-				pvmlogperror("work() select");
+				pvmlogperror("pl_startup() select");
 				pvmbailout(0);
 			}
 		}
@@ -915,57 +1085,34 @@ pl_startup(htp)
 					sp->s_len += n;
 					if (sp->s_len >= sizeof(sp->s_buf)) {
 						sprintf(pvmtxt, "pl_startup() pvmd@%s: big read\n",
-								sp->s_hostd->hd_name);
+								sp->s_hst->h_name);
 						pvmlogerror(pvmtxt);
-						sp->s_hostd->hd_err = PvmCantStart;
-						sp2 = sp->s_rlink;
-						close_slot(sp);
-						sp = sp2;
-						continue;
+						sp->s_hst->h_result = STRALLOC("PvmCantStart");
 					}
 					sp->s_buf[sp->s_len] = 0;
-					if (CINDEX(sp->s_buf + sp->s_len - n, '\n')) {
+					if (p = CINDEX(sp->s_buf + sp->s_len - n, '\n')) {
 						if (debugmask & PDMSTARTUP) {
 							sprintf(pvmtxt, "pvmd@%s: %s",
-									sp->s_hostd->hd_name, sp->s_buf);
+									sp->s_hst->h_name, sp->s_buf);
 							pvmlogerror(pvmtxt);
 						}
-						ac = sizeof(av)/sizeof(av[0]);
-						if (crunchzap(sp->s_buf, &ac, av) || ac != 4) {
-							pvmlogerror("pl_startup() expected version\n");
-							sp->s_hostd->hd_err = PvmCantStart;
-
-						} else {
-							ver = atoi(av[0]);
-							if (ver != DDPROTOCOL) {
-								sprintf(pvmtxt, "pl_startup() host %s d-d protocol mismatch (%d/%d)\n",
-										sp->s_hostd->hd_name, ver, DDPROTOCOL);
-								pvmlogerror(pvmtxt);
-								sp->s_hostd->hd_err = PvmBadVersion;
-							} else {
-
-								sp->s_hostd->hd_arch = STRALLOC(av[1]);
-								hex_inadport(av[2], &sp->s_hostd->hd_sad);
-								sp->s_hostd->hd_mtu = atoi(av[3]);
-							}
-						}
-						sp2 = sp->s_rlink;
-						close_slot(sp);
-						sp = sp2;
-						continue;
+						*p = 0;
+						sp->s_hst->h_result = STRALLOC(sp->s_buf);
 					}
 
 				} else {
 					if (n) {
 						sprintf(pvmtxt, "pl_startup() pvmd@%s",
-								sp->s_hostd->hd_name);
+								sp->s_hst->h_name);
 						pvmlogperror(pvmtxt);
 					} else {
 						sprintf(pvmtxt, "pl_startup() pvmd@%s: EOF\n",
-								sp->s_hostd->hd_name);
+								sp->s_hst->h_name);
 						pvmlogerror(pvmtxt);
 					}
-					sp->s_hostd->hd_err = PvmCantStart;
+					sp->s_hst->h_result = STRALLOC("PvmCantStart");
+				}
+				if (sp->s_hst->h_result) {
 					sp2 = sp->s_rlink;
 					close_slot(sp);
 					sp = sp2;
@@ -981,7 +1128,7 @@ pl_startup(htp)
 					char *p = ebuf, *q, c;
 
 					ebuf[n] = 0;
-					sprintf(pvmtxt, "pvmd@%s: ", sp->s_hostd->hd_name);
+					sprintf(pvmtxt, "pvmd@%s: ", sp->s_hst->h_name);
 					q = pvmtxt + strlen(pvmtxt);
 					while (c = *p++ & 0x7f) {
 						if (isprint(c))
@@ -1010,15 +1157,13 @@ pl_startup(htp)
 phase1(sp)
 	struct slot *sp;
 {
-	struct hostd *hp;
-	int byrsh;
+	struct hst *hp;
 	char *hn;
-	char *av[16];			/* for rsh args and reply parsing */
+	char *av[16];			/* for rsh args */
 	int ac;
 	char buf[512];
 	int pid = -1;			/* pid of rsh */
 	char *p;
-	int ver;
 
 #ifndef NOREXEC
 	struct servent *se;
@@ -1034,27 +1179,18 @@ phase1(sp)
 	}
 #endif
 
-	hp = sp->s_hostd;
-	hn = hp->hd_name;
+	hp = sp->s_hst;
+	hn = hp->h_name;
 	sp->s_rfd = sp->s_wfd = sp->s_efd = -1;
 
 	/*
 	* XXX manual startup hack... this is if we can't use rexec or rsh
 	*/
 
-	if (hp->hd_flag & HF_MANUAL) {
+	if (hp->h_flag & HST_MANUAL) {
 		fprintf(stderr, "*** Manual startup ***\n");
 		fprintf(stderr, "Login to \"%s\" and type:\n", hn);
-		fprintf(stderr, "%s -S -d%x -n%s %d %s %d",
-				hp->hd_dpath ? hp->hd_dpath : pvmgetrpvmd(),
-				debugmask,
-				hn,
-				hosts->ht_master,
-				inadport_hex(&hosts->ht_hosts[hosts->ht_master]->hd_sad),
-				hosts->ht_hosts[hosts->ht_master]->hd_mtu);
-		fprintf(stderr, " %d %s\n",
-				((hp->hd_hostpart & tidhmask) >> (ffs(tidhmask) - 1)),
-				inadport_hex(&hp->hd_sad));
+		fprintf(stderr, "%s\n", hp->h_cmd);
 
 	/* get version */
 
@@ -1065,24 +1201,10 @@ phase1(sp)
 			pvmlogerror(pvmtxt);
 			goto oops;
 		}
-		ac = sizeof(av)/sizeof(av[0]);
-		if (crunchzap(buf, &ac, av) || ac != 4) {
-			pvmlogerror("slave_exec() expected version\n");
-			goto oops;
-		}
-		ver = atoi(av[0]);
-		if (ver != DDPROTOCOL) {
-			sprintf(pvmtxt, "slave_exec() host %s d-d protocol mismatch (%d/%d)\n",
-					hn, ver, DDPROTOCOL);
-			pvmlogerror(pvmtxt);
-			hp->hd_err = PvmBadVersion;
-			goto oops2;
-		}
-
-		hp->hd_arch = STRALLOC(av[1]);
-		hex_inadport(av[2], &hp->hd_sad);
-		hp->hd_mtu = atoi(av[3]);
-
+		p = buf + strlen(buf) - 1;
+		if (*p == '\n')
+			*p = 0;
+		hp->h_result = STRALLOC(buf);
 		fprintf(stderr, "Thanks\n");
 		fflush(stderr);
 		return 0;
@@ -1092,9 +1214,7 @@ phase1(sp)
 	* XXX end manual startup hack
 	*/
 
-	byrsh = !(hp->hd_flag & HF_ASKPW);
-
-	if (byrsh) {			/* use rsh to start */
+	if (!(hp->h_flag & HST_PASSWORD)) {		/* use rsh to start */
 		int wpfd[2], rpfd[2], epfd[2];
 		int i;
 
@@ -1138,40 +1258,17 @@ phase1(sp)
 			ac = 0;
 			av[ac++] = RSHCOMMAND;
 			av[ac++] = hn;
-			if (hp->hd_login) {
+			if (hp->h_login) {
 				av[ac++] = "-l";
-				av[ac++] = hp->hd_login;
+				av[ac++] = hp->h_login;
 			}
-			av[ac++] = hp->hd_dpath ? hp->hd_dpath : pvmgetrpvmd();
-			av[ac++] = "-s";
-			(void)sprintf(buf, "-d%x", debugmask);
-			av[ac++] = STRALLOC(buf);
-
-			(void)sprintf(buf, "-n%s", hn);
-			av[ac++] = STRALLOC(buf);
-
-			/* master host index */
-			sprintf(buf, "%d", hosts->ht_master);
-			av[ac++] = STRALLOC(buf);
-
-			/* master host sockaddr */
-			p = inadport_hex(&hosts->ht_hosts[hosts->ht_master]->hd_sad);
-			av[ac++] = STRALLOC(p);
-
-			/* master host mtu */
-			sprintf(buf, "%d", hosts->ht_hosts[hosts->ht_master]->hd_mtu);
-			av[ac++] = STRALLOC(buf);
-
-			/* this slave index */
-			sprintf(buf, "%d",
-					((hp->hd_hostpart & tidhmask) >> (ffs(tidhmask) - 1)));
-			av[ac++] = STRALLOC(buf);
-
-			/* this slave sockaddr */
-			p = inadport_hex(&hp->hd_sad);
-			av[ac++] = STRALLOC(p);
-
+			av[ac++] = hp->h_cmd;
 			av[ac++] = 0;
+			if (debugmask & PDMSTARTUP) {
+				for (ac = 0; av[ac]; ac++)
+					fprintf(stderr, "av[%d]=\"%s\" ", ac, av[ac]);
+				fputc('\n', stderr);
+			}
 			execvp(av[0], av);
 			fputs("phase1() execvp failed\n", stderr);
 			fflush(stderr);
@@ -1191,23 +1288,13 @@ phase1(sp)
 		pvmlogerror(pvmtxt);
 		goto oops;
 #else
-		(void)sprintf(buf, "%s -s -d%x -n%s %d %s %d",
-				hp->hd_dpath ? hp->hd_dpath : pvmgetrpvmd(),
-				debugmask,
-				hn,
-				hosts->ht_master,
-				inadport_hex(&hosts->ht_hosts[hosts->ht_master]->hd_sad),
-				hosts->ht_hosts[hosts->ht_master]->hd_mtu);
-		(void)sprintf(buf + strlen(buf), " %d %s\n",
-				((hp->hd_hostpart & tidhmask) >> (ffs(tidhmask) - 1)),
-				inadport_hex(&hp->hd_sad));
 		if (debugmask & PDMSTARTUP) {
-			sprintf(pvmtxt, "phase1() rexec \"%s\"\n", buf);
+			sprintf(pvmtxt, "phase1() rexec \"%s\"\n", hp->h_cmd);
 			pvmlogerror(pvmtxt);
 		}
 		if ((sp->s_wfd = sp->s_rfd = rexec(&hn, execport,
-				(hp->hd_login ? hp->hd_login : username),
-				(char*)0, buf, &sp->s_efd))
+				(hp->h_login ? hp->h_login : username),
+				(char*)0, hp->h_cmd, &sp->s_efd))
 		== -1) {
 			sprintf(pvmtxt, "phase1() rexec failed for host %s\n", hn);
 			pvmlogerror(pvmtxt);
@@ -1218,8 +1305,7 @@ phase1(sp)
 	return 0;
 
 oops:
-	hp->hd_err = PvmCantStart;
-oops2:
+	hp->h_result = STRALLOC("PvmCantStart");
 	if (sp->s_wfd != -1)
 		close(sp->s_wfd);
 	if (sp->s_rfd != -1)
@@ -1231,359 +1317,3 @@ oops2:
 }
 
 
-#else	/*RSHNPLL > 1*/
-
-/******************************************
-*  this is the old (serial) startup code  *
-*                                         *
-******************************************/
-
-/*	eread()
-*
-*	Read an fd until \n is reached.  Also pass input from a second fd
-*	to logfile.
-*/
-
-int
-eread(rfd, buf, len, efd, hname)
-	int rfd;			/* fd to read from */
-	char *buf;			/* buffer for data */
-	int len;			/* buffer length */
-	int efd;			/* alt fd to pass to stderr or -1 if none */
-	char *hname;		/* hostname we're talking to */
-{
-	int got = 0;			/* length written into buf */
-	fd_set rfds, the_fds;
-	struct timeval tout;
-	char lbuf[256];			/* for std err */
-	int n;
-	int i;
-	int mfd = max(rfd, efd) + 1;
-
-	FD_ZERO(&the_fds);
-	FD_SET(rfd, &the_fds);
-	if (efd >= 0)
-		FD_SET(efd, &the_fds);
-	tout.tv_sec = RSHTIMEOUT;	/* max to get response */
-	tout.tv_usec = 0;
-
-	while (1) {
-		do {
-			rfds = the_fds;
-			if ((n = select(mfd, &rfds, (fd_set*)0, (fd_set*)0, &tout)) == -1) {
-				pvmlogperror("eread() select");
-				if (errno != EINTR)
-					return -1;
-			}
-		} while (n == -1);
-		if (!n) {
-	/*
-	* select timeout - no response from slave pvmd
-	*/
-			sprintf(pvmtxt, "eread() no response from pvmd@%s in %d seconds\n",
-					hname, RSHTIMEOUT);
-			pvmlogerror(pvmtxt);
-			return 0;
-		}
-
-		if (FD_ISSET(rfd, &rfds)) {
-			if ((n = read(rfd, buf, len)) == -1) {
-				pvmlogperror("eread() read");
-				return 0;
-			}
-			if (!n)
-				return got;
-			got += n;
-			for (i = 0; i < n; i++)
-				if (buf[i] == '\n')
-					return got;
-			buf += n;
-			len -= n;
-		}
-
-		if (efd >= 0 && FD_ISSET(efd, &rfds)) {
-	/*
-	* response on standard error.  log prefixed by remote's host name.
-	*/
-			if ((n = read(efd, lbuf, sizeof(lbuf)-1)) > 0) {
-				char *p = lbuf, *q, c;
-
-				lbuf[n] = 0;
-				sprintf(pvmtxt, "eread() pvmd@%s: ", hname);
-				q = pvmtxt + strlen(pvmtxt);
-				while (c = *p++ & 0x7f) {
-					if (isprint(c))
-						*q++ = c;
-
-					else {
-						*q++ = '^';
-						*q++ = (c + '@') & 0x7f;
-					}
-				}
-				*q++ = '\n';
-				*q = 0;
-				pvmlogerror(pvmtxt);
-			}
-		}
-	}
-}
-
-
-/*	slave_exec()
-*
-*	Exec a new slave pvmd and do first-stage configuration.
-*	Returns with hp->hd_err set if error occurred, else
-*	other fields of hp filled in with config info.
-*/
-
-slave_exec(hp)
-	struct hostd *hp;
-{
-	int wfd = -1;			/* slave fd[0] */
-	int rfd = -1;			/* slave fd[1] */
-	int efd = -1;			/* slave fd[2] */
-	char *hn = hp->hd_name;
-	char *av[16];			/* for rsh args and reply parsing */
-	int ac;
-	char buf[512];
-	int n;
-	int ver;
-	int byrsh = !(hp->hd_flag & HF_ASKPW);
-	int pid = -1;			/* pid of rsh */
-	char *p;
-
-#ifndef NOREXEC
-	struct servent *se;
-	static u_short execport = 0;
-
-	if (!execport) {
-		if (!(se = getservbyname("exec", "tcp"))) {
-			sprintf(pvmtxt, "slave_exec() can't getservbyname(): %s\n", "exec");
-			pvmbailout(0);
-		}
-		execport = se->s_port;
-		endservent();
-	}
-#endif
-
-	/*
-	* XXX gratuitous hack... this is if we can't use rexec or rsh
-	* XXX some day we'll have real network auth functions
-	*/
-
-	if (hp->hd_flag & HF_MANUAL) {
-		fprintf(stderr, "*** Manual startup ***\n");
-		fprintf(stderr, "Login to \"%s\" and type:\n", hp->hd_name);
-		fprintf(stderr, "%s -S -d%x -n%s %d %s %d",
-				hp->hd_dpath ? hp->hd_dpath : pvmgetrpvmd(),
-				debugmask,
-				hn,
-				hosts->ht_master,
-				inadport_hex(&hosts->ht_hosts[hosts->ht_master]->hd_sad),
-				hosts->ht_hosts[hosts->ht_master]->hd_mtu);
-		fprintf(stderr, " %d %s\n",
-				((hp->hd_hostpart & tidhmask) >> (ffs(tidhmask) - 1)),
-				inadport_hex(&hp->hd_sad));
-
-	/* get version */
-
-		fprintf(stderr, "Type response: ");
-		fflush(stderr);
-		if (!(fgets(buf, sizeof(buf), stdin))) {
-			sprintf(pvmtxt, "host %s read error\n", hn);
-			pvmlogerror(pvmtxt);
-			goto oops;
-		}
-		ac = sizeof(av)/sizeof(av[0]);
-		if (crunchzap(buf, &ac, av) || ac != 4) {
-			pvmlogerror("slave_exec() expected version\n");
-			goto oops;
-		}
-		ver = atoi(av[0]);
-		if (ver != DDPROTOCOL) {
-			sprintf(pvmtxt, "slave_exec() host %s d-d protocol mismatch (%d/%d)\n",
-					hn, ver, DDPROTOCOL);
-			pvmlogerror(pvmtxt);
-			hp->hd_err = PvmBadVersion;
-			goto done;
-		}
-
-		hp->hd_arch = STRALLOC(av[1]);
-		hex_inadport(av[2], &hp->hd_sad);
-		hp->hd_mtu = atoi(av[3]);
-
-		fprintf(stderr, "Thanks\n");
-		fflush(stderr);
-		goto done;
-	}
-
-	/*
-	* XXX end manual startup hack
-	*/
-
-	if (byrsh) {			/* use rsh to start */
-		int wpfd[2], rpfd[2], epfd[2];
-		int i;
-
-		if (debugmask & PDMSTARTUP) {
-			sprintf(pvmtxt, "slave_exec() trying rsh to %s\n", hn);
-			pvmlogerror(pvmtxt);
-		}
-
-	/* fork an rsh to startup the slave pvmd */
-
-#ifdef	IMA_TITN
-		if (socketpair(AF_UNIX, SOCK_STREAM, 0, wpfd) == -1
-		|| socketpair(AF_UNIX, SOCK_STREAM, 0, rpfd) == -1
-		|| socketpair(AF_UNIX, SOCK_STREAM, 0, epfd) == -1) {
-			pvmlogperror("slave_exec() socketpair");
-			goto oops;
-		}
-#else
-		if (pipe(wpfd) == -1 || pipe(rpfd) == -1 || pipe(epfd) == -1) {
-			pvmlogperror("slave_exec() pipe");
-			goto oops;
-		}
-#endif
-
-		if (debugmask & PDMSTARTUP) {
-			sprintf(pvmtxt, "slave_exec() pipes: %d %d %d %d %d %d\n",
-					wpfd[0], wpfd[1], rpfd[0], rpfd[1], epfd[0], epfd[1]);
-			pvmlogerror(pvmtxt);
-		}
-
-		if ((pid = fork()) == -1) {
-			pvmlogperror("slave_exec() fork");
-			pvmbailout(0);
-		}
-		if (!pid) {
-			(void)dup2(wpfd[0], 0);
-			(void)dup2(rpfd[1], 1);
-			(void)dup2(epfd[1], 2);
-			for (i = getdtablesize(); --i > 2; )
-				(void)close(i);
-			ac = 0;
-			av[ac++] = RSHCOMMAND;
-			av[ac++] = hn;
-			if (hp->hd_login) {
-				av[ac++] = "-l";
-				av[ac++] = hp->hd_login;
-			}
-			av[ac++] = hp->hd_dpath ? hp->hd_dpath : pvmgetrpvmd();
-			av[ac++] = "-s";
-			(void)sprintf(buf, "-d%x", debugmask);
-			av[ac++] = STRALLOC(buf);
-
-			(void)sprintf(buf, "-n%s", hn);
-			av[ac++] = STRALLOC(buf);
-
-			/* master host index */
-			sprintf(buf, "%d", hosts->ht_master);
-			av[ac++] = STRALLOC(buf);
-
-			/* master host sockaddr */
-			p = inadport_hex(&hosts->ht_hosts[hosts->ht_master]->hd_sad);
-			av[ac++] = STRALLOC(p);
-
-			/* master host mtu */
-			sprintf(buf, "%d", hosts->ht_hosts[hosts->ht_master]->hd_mtu);
-			av[ac++] = STRALLOC(buf);
-
-			/* this slave index */
-			sprintf(buf, "%d",
-					((hp->hd_hostpart & tidhmask) >> (ffs(tidhmask) - 1)));
-			av[ac++] = STRALLOC(buf);
-
-			/* this slave sockaddr */
-			p = inadport_hex(&hp->hd_sad);
-			av[ac++] = STRALLOC(p);
-
-			av[ac++] = 0;
-			execvp(av[0], av);
-			fputs("slave_exec() execvp failed\n", stderr);
-			fflush(stderr);
-			_exit(1);
-		}
-		(void)close(wpfd[0]);
-		(void)close(rpfd[1]);
-		(void)close(epfd[1]);
-		wfd = wpfd[1];
-		rfd = rpfd[0];
-		efd = epfd[0];
-
-	} else {		/* use rexec to start */
-
-#ifdef NOREXEC
-		sprintf(pvmtxt, "slconfg() sorry, no rexec()\n");
-		pvmlogerror(pvmtxt);
-		goto oops;
-#else
-		(void)sprintf(buf, "%s -s -d%x -n%s %d %s %d",
-				hp->hd_dpath ? hp->hd_dpath : pvmgetrpvmd(),
-				debugmask,
-				hn,
-				hosts->ht_master,
-				inadport_hex(&hosts->ht_hosts[hosts->ht_master]->hd_sad),
-				hosts->ht_hosts[hosts->ht_master]->hd_mtu);
-		(void)sprintf(buf + strlen(buf), " %d %s\n",
-				((hp->hd_hostpart & tidhmask) >> (ffs(tidhmask) - 1)),
-				inadport_hex(&hp->hd_sad));
-		if (debugmask & PDMSTARTUP) {
-			sprintf(pvmtxt, "slave_exec() rexec \"%s\"\n", buf);
-			pvmlogerror(pvmtxt);
-		}
-		if ((wfd = rfd = rexec(&hn, execport,
-				(hp->hd_login ? hp->hd_login : username),
-				(char*)0, buf, &efd))
-		== -1) {
-			sprintf(pvmtxt, "slave_exec() rexec failed for host %s\n", hn);
-			pvmlogerror(pvmtxt);
-			goto oops;
-		}
-#endif
-	}
-
-	/* get its version */
-
-	if ((n = eread(rfd, buf, sizeof(buf)-1, efd, hn)) == -1) {
-		sprintf(pvmtxt, "host %s read error\n", hn);
-		pvmlogerror(pvmtxt);
-		goto oops;
-	}
-	buf[n] = 0;
-	if (debugmask & PDMSTARTUP)
-		pvmlogerror(buf);
-	ac = sizeof(av)/sizeof(av[0]);
-	if (crunchzap(buf, &ac, av) || ac != 4) {
-		pvmlogerror("slave_exec() expected version\n");
-		goto oops;
-	}
-
-	ver = atoi(av[0]);
-	if (ver != DDPROTOCOL) {
-		sprintf(pvmtxt, "slave_exec() host %s d-d protocol mismatch (%d/%d)\n",
-				hn, ver, DDPROTOCOL);
-		pvmlogerror(pvmtxt);
-		hp->hd_err = PvmBadVersion;
-		goto done;
-	}
-
-	hp->hd_arch = STRALLOC(av[1]);
-	hex_inadport(av[2], &hp->hd_sad);
-	hp->hd_mtu = atoi(av[3]);
-	goto done;
-
-oops:
-	if (pid != -1)
-		(void)kill(pid, SIGTERM);
-	hp->hd_err = PvmCantStart;
-done:
-	close(wfd);
-	close(rfd);
-	close(efd);
-	wait(0);
-	return;
-}
-#endif	/*RSHNPLL > 1*/
-
-
