@@ -1,6 +1,6 @@
 
 /*
- *           PVM 3.2:  Parallel Virtual Machine System 3.2
+ *         PVM version 3.3:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
@@ -22,8 +22,8 @@
  * purpose.  This software is provided ``as is'' without express or
  * implied warranty.
  *
- * PVM 3.2 was funded in part by the U.S. Department of Energy, the
- * National Science Foundation and the State of Tennessee.
+ * PVM version 3 was funded in part by the U.S. Department of Energy,
+ * the National Science Foundation and the State of Tennessee.
  */
 
 /*
@@ -32,6 +32,19 @@
  *	Task descriptors.
  *
 $Log: task.c,v $
+ * Revision 1.5  1994/11/08  15:40:59  manchek
+ * shared memory cleanup.
+ * check if wa_tid is zero before sending message in task_cleanup
+ *
+ * Revision 1.4  1994/10/15  19:32:49  manchek
+ * added log in task_free()
+ *
+ * Revision 1.3  1994/06/03  20:38:28  manchek
+ * version 3.3.0
+ *
+ * Revision 1.2  1993/11/30  15:55:25  manchek
+ * task_free() deletes auth file if one exists
+ *
  * Revision 1.1  1993/08/30  23:26:51  manchek
  * Initial revision
  *
@@ -48,7 +61,9 @@ $Log: task.c,v $
 #endif
 
 #include "global.h"
+#include "fromlib.h"
 #include "ddpro.h"
+#include <pvmsdpro.h>
 #include "protoglarp.h"
 #include "mesg.h"
 #include "pkt.h"
@@ -66,7 +81,12 @@ $Log: task.c,v $
 
 extern void pvmbailout();
 
+extern int busyadding;				/* from ddpro.c */
+extern int debugmask;				/* from pvmd.c */
+extern int hostertid;				/* from pvmd.c */
 extern int myhostpart;				/* from pvmd.c */
+extern int mytid;					/* from pvmd.c */
+extern int taskertid;				/* from pvmd.c */
 extern int tidlmask;				/* from pvmd.c */
 extern struct waitc *waitlist;		/* from waitc.c */
 
@@ -79,7 +99,7 @@ void task_dump();
  **           **
  ***************/
 
-static char rcsid[] = "$Id: task.c,v 1.1 1993/08/30 23:26:51 manchek Exp $";
+static char rcsid[] = "$Id: task.c,v 1.5 1994/11/08 15:40:59 manchek Exp $";
 static char pvmtxt[512];		/* scratch for error log */
 
 
@@ -189,6 +209,19 @@ void
 task_free(tp)
 	struct task *tp;
 {
+	struct timeval now;
+	struct mesg *mp;
+
+	if (debugmask & PDMTASK) {
+		sprintf(pvmtxt, "task_free() t%x\n", tp->t_tid);
+		pvmlogerror(pvmtxt);
+	}
+#ifdef SHMEM
+	/* XXX this is inside out - mpp_free should call task_free.
+	   XXX but for now task_free is what's called.
+	   XXX this will change in the portable processor interface cleanup. */
+	mpp_free(tp->t_tid);
+#endif
 	if (tp->t_plink && tp->t_prlink) {
 		LISTDELETE(tp, t_plink, t_prlink);
 	}
@@ -203,26 +236,43 @@ task_free(tp)
 		pk_free(tp->t_txq);
 	if (tp->t_wait)
 		wait_delete(tp->t_wait);
-	if (tp->t_authnam)
+	if (tp->t_authnam) {
+		(void)unlink(tp->t_authnam);
 		PVM_FREE(tp->t_authnam);
+	}
 	if (tp->t_sock != -1) {
 		wrk_fds_delete(tp->t_sock, 3);
 		(void)close(tp->t_sock);
 	}
 	if (tp->t_out != -1) {
-		struct mesg *mp;
-
 		wrk_fds_delete(tp->t_out, 1);
 		(void)close(tp->t_out);
-		if (tp->t_outtid) {
-			mp = mesg_new(0);
-			mp->m_cod = tp->t_outcod;
-			mp->m_dst = tp->t_outtid;
-			pkint(mp, tp->t_tid);
-			pkint(mp, 0);
-			sendmessage(mp);
-			tp->t_outtid = 0;
-		}
+	}
+	if (tp->t_outtid) {
+		mp = mesg_new(0);
+		mp->m_cod = tp->t_outcod;
+		mp->m_dst = tp->t_outtid;
+		pkint(mp, tp->t_tid);
+		pkint(mp, 0);
+		sendmessage(mp);
+		tp->t_outtid = 0;
+	}
+	if (tp->t_trctid) {
+		mp = mesg_new(0);
+		mp->m_cod = tp->t_trccod;
+		mp->m_dst = tp->t_trctid;
+		gettimeofday(&now, (struct timezone*)0);
+		pkint(mp, (int)now.tv_sec);
+		pkint(mp, (int)now.tv_usec);
+		pkint(mp, tp->t_tid);
+		pkint(mp, TEV_ENDTASK);
+		pkint(mp, tp->t_status);
+		pkint(mp, (int)tp->t_utime.tv_sec);
+		pkint(mp, (int)tp->t_utime.tv_usec);
+		pkint(mp, (int)tp->t_stime.tv_sec);
+		pkint(mp, (int)tp->t_stime.tv_usec);
+		sendmessage(mp);
+		tp->t_trctid = 0;
 	}
 	if (tp->t_authfd != -1)
 		(void)close(tp->t_authfd);
@@ -311,28 +361,69 @@ task_cleanup(tp)
 {
 	struct mesg *mp;
 	struct waitc *wp, *wp2;
+	struct task *tp2;
 
 	/* notify anyone who asked */
 
 	if (tp->t_tid) {
+		if (debugmask & PDMTASK) {
+			sprintf(pvmtxt, "task_cleanup() t%x\n", tp->t_tid);
+			pvmlogerror(pvmtxt);
+		}
 		for (wp = waitlist->wa_link; wp != waitlist; wp = wp->wa_link) {
+
+	/* waits depending on this task */
+
 			if (wp->wa_on == tp->t_tid) {
 				switch (wp->wa_kind) {
 
-				case WT_TASKX:
-					mp = mesg_new(0);
-					mp->m_dst = wp->wa_tid;
-					if (TIDISTASK(mp->m_dst))
-						mp->m_cod = wp->wa_dep;
-					else {
-						mp->m_cod = DM_NOTIFYACK;
-						mp->m_wid = wp->wa_dep;
+				case WT_HOSTSTART:
+					busyadding = 0;
+					free_waitc_add((struct waitc_add *)wp->wa_spec);
+					pkint(wp->wa_mesg, PvmDSysErr);
+					sendmessage(wp->wa_mesg);
+					wp->wa_mesg = 0;
+					break;
+
+				case WT_TASKSTART:
+					if (wp->wa_tid) {
+						if (debugmask & PDMTASK) {
+							sprintf(pvmtxt,
+								"task_cleanup() tasker t%x takes t%x with it\n",
+								tp->t_tid, wp->wa_tid);
+							pvmlogerror(pvmtxt);
+						}
+						if (tp2 = task_find(wp->wa_tid)) {
+							wp->wa_tid = 0;
+							task_cleanup(tp2);
+							task_free(tp2);
+						}
 					}
-					pkint(mp, tp->t_tid);
-					sendmessage(mp);
+					break;
+
+				case WT_TASKX:
+					if (wp->wa_tid) {
+						mp = mesg_new(0);
+						mp->m_dst = wp->wa_tid;
+						if (TIDISTASK(mp->m_dst))
+							mp->m_cod = wp->wa_dep;
+						else {
+							mp->m_cod = DM_NOTIFYACK;
+							mp->m_wid = wp->wa_dep;
+						}
+						pkint(mp, tp->t_tid);
+						sendmessage(mp);
+					}
+					break;
+
+				case WT_HOSTA:
 					break;
 
 				default:
+					sprintf(pvmtxt,
+							"task_cleanup() can't deal with wait kind %d\n",
+							wp->wa_kind);
+					pvmlogerror(pvmtxt);
 					break;
 				}
 				wp2 = wp;
@@ -340,8 +431,63 @@ task_cleanup(tp)
 				wait_delete(wp2);
 				continue;
 			}
-			if (wp->wa_tid == tp->t_tid)
-				wp->wa_tid = 0;				/* in case tid gets recycled */
+
+	/* waits this task was waiting on */
+
+			if (wp->wa_tid == tp->t_tid) {
+				switch (wp->wa_kind) {
+
+				case WT_TASKSTART:
+					wp2 = wp;				/* some kinds we can toss now */
+					wp = wp->wa_rlink;
+					wait_delete(wp2);
+					break;
+
+				default:
+					wp->wa_tid = 0;			/* in case tid gets recycled */
+					break;
+				}
+			}
+		}
+
+		/* notify the scheduler */
+
+		if (tp->t_sched) {
+			mp = mesg_new(0);
+			mp->m_dst = tp->t_sched;
+			mp->m_cod = SM_TASKX;
+			if (debugmask & PDMSCHED) {
+				sprintf(pvmtxt, "task_cleanup() taskx to t%x status = 0x%x\n",
+						tp->t_sched, tp->t_status);
+				pvmlogerror(pvmtxt);
+			}
+			pkint(mp, tp->t_tid);
+			pkint(mp, tp->t_status);
+			pkint(mp, (int)tp->t_utime.tv_sec);
+			pkint(mp, (int)tp->t_utime.tv_usec);
+			pkint(mp, (int)tp->t_stime.tv_sec);
+			pkint(mp, (int)tp->t_stime.tv_usec);
+			sendmessage(mp);
+		}
+
+		/* check if it's the hoster */
+
+		if (tp->t_tid == hostertid) {
+			if (debugmask & (PDMTASK|PDMSTARTUP)) {
+				sprintf(pvmtxt, "task_cleanup() unreg hoster t%x\n", tp->t_tid);
+				pvmlogerror(pvmtxt);
+			}
+			hostertid = 0;
+		}
+
+		/* check if it's the tasker */
+
+		if (tp->t_tid == taskertid) {
+			if (debugmask & PDMTASK) {
+				sprintf(pvmtxt, "task_cleanup() unreg tasker t%x\n", tp->t_tid);
+				pvmlogerror(pvmtxt);
+			}
+			taskertid = 0;
 		}
 	}
 
@@ -369,9 +515,9 @@ task_dump()
 				tp->t_flag, tp->t_pid,
 				tp->t_outtid, tp->t_trctid);
 		pvmlogerror(pvmtxt);
-		pvmlogerror(" txq:     pkt      src      dst     flag      len      ofs\n");
+		pvmlogerror(" txq:pkt      src      dst   flag      len      ofs\n");
 		for (pp = tp->t_txq->pk_link; pp != tp->t_txq; pp = pp->pk_link) {
-			sprintf(pvmtxt, "     %08x %8x %8x %8x %8d %8d\n",
+			sprintf(pvmtxt, "%08x %8x %8x %6x %8d %8d\n",
 					pp,
 					pp->pk_src,
 					pp->pk_dst,
