@@ -1,6 +1,6 @@
 
 /*
- *           PVM 3.2:  Parallel Virtual Machine System 3.2
+ *         PVM version 3.3:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
@@ -22,8 +22,8 @@
  * purpose.  This software is provided ``as is'' without express or
  * implied warranty.
  *
- * PVM 3.2 was funded in part by the U.S. Department of Energy, the
- * National Science Foundation and the State of Tennessee.
+ * PVM version 3 was funded in part by the U.S. Department of Energy,
+ * the National Science Foundation and the State of Tennessee.
  */
 
 /*
@@ -32,6 +32,12 @@
  *	Task descriptors.
  *
 $Log: task.h,v $
+ * Revision 1.3  1994/10/15  19:33:47  manchek
+ * added TF_FORKD flag
+ *
+ * Revision 1.2  1994/06/03  20:38:28  manchek
+ * version 3.3.0
+ *
  * Revision 1.1  1993/08/30  23:26:52  manchek
  * Initial revision
  *
@@ -65,13 +71,21 @@ struct task {
 	int t_outcod;
 	int t_trctid;				/* dst and code for trace data */
 	int t_trccod;
+	int	t_sched;				/* scheduler responsible for this task */
+	int t_status;				/* unix exit status and code */
+	struct timeval t_utime;		/* user time used */
+	struct timeval t_stime;		/* system time used */
 };
 
 /* t_flag bits */
 
+#define	TF_FORKD	2			/* task process is child of pvmd */
 #define	TF_CONN		4			/* task is connected */
 #define	TF_AUTH		8			/* task needs to auth */
 #define	TF_CLOSE	16			/* should delete context after flushing msgs */
+#define	TF_ISSCHED	0x100		/* task is scheduler */
+#define	TF_ISHOSTER	0x200		/* task is hoster */
+#define	TF_ISTASKER	0x400		/* task is tasker */
 
 #define	ISLOCLTASK(x)	(((x)&tidhmask)==(hosts->ht_local<<(ffs(tidhmask)-1)))
 
