@@ -1,6 +1,6 @@
 
 /*
- *           PVM 3.2:  Parallel Virtual Machine System 3.2
+ *         PVM version 3.3:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
@@ -22,8 +22,8 @@
  * purpose.  This software is provided ``as is'' without express or
  * implied warranty.
  *
- * PVM 3.2 was funded in part by the U.S. Department of Energy, the
- * National Science Foundation and the State of Tennessee.
+ * PVM version 3 was funded in part by the U.S. Department of Energy,
+ * the National Science Foundation and the State of Tennessee.
  */
 
 /*
@@ -32,6 +32,23 @@
  *	Entry points for messages from local tasks.
  *
 $Log: tdpro.c,v $
+ * Revision 1.6  1994/10/15  19:37:46  manchek
+ * cast message tags for comparison as integers.
+ * check newhosts when deleting host.
+ * unset TF_FORKD if task reconnects with a different pid
+ *
+ * Revision 1.5  1994/06/21  18:30:00  manchek
+ * subscript arith in tmname() broke with opt
+ *
+ * Revision 1.4  1994/06/04  21:45:54  manchek
+ * misc
+ *
+ * Revision 1.3  1994/06/03  20:38:31  manchek
+ * version 3.3.0
+ *
+ * Revision 1.2  1993/11/30  15:56:23  manchek
+ * tm_conn() complains if it can't write t-auth file (fs full?)
+ *
  * Revision 1.1  1993/08/30  23:26:52  manchek
  * Initial revision
  *
@@ -55,6 +72,7 @@ $Log: tdpro.c,v $
 #include "fromlib.h"
 #include "tdpro.h"
 #include "ddpro.h"
+#include <pvmsdpro.h>
 #include "protoglarp.h"
 #include "pvmalloc.h"
 #include "mesg.h"
@@ -68,6 +86,20 @@ $Log: tdpro.c,v $
 #define	min(a,b)	((a)<(b)?(a):(b))
 #endif
 
+#ifdef	NOTMPNAM
+#define	TMPNAMFUN(x)	pvmtmpnam(x)
+#define	LEN_OF_TMP_NAM	32
+char *pvmtmpnam();
+
+#else	/*NOTMPNAM*/
+#define	TMPNAMFUN(x)	tmpnam(x)
+#ifdef	L_tmpnam
+#define	LEN_OF_TMP_NAM	L_tmpnam
+#else
+#define	LEN_OF_TMP_NAM	64
+#endif
+#endif	/*NOTMPNAM*/
+
 
 /***************
  **  Globals  **
@@ -76,18 +108,25 @@ $Log: tdpro.c,v $
 
 extern void task_dump();
 char *debug_flags();
+char *inadport_hex();
 
 extern int debugmask;				/* from pvmd.c */
+extern int hostertid;				/* from pvmd.c */
 extern struct htab *hosts;			/* from pvmd.c */
 extern struct task *locltasks;		/* from task.c */
+extern char *myarchname;			/* from pvmd.c */
 extern int myndf;					/* from pvmd.c */
 extern int myhostpart;				/* from pvmd.c */
 extern int mytid;					/* from pvmd.c */
+extern struct htab *newhosts;		/* from pvmd.c */
 extern int nopax;					/* from pvmd.c */
 extern int ourudpmtu;				/* from pvmd.c */
 extern int runstate;				/* from pvmd.c */
+extern int pvmschedtid;				/* from pvmd.c */
+extern int taskertid;				/* from pvmd.c */
 extern int tidhmask;				/* from pvmd.c */
 extern int tidlmask;				/* from pvmd.c */
+extern struct waitc *waitlist;		/* from waitc.c */
 
 
 /***************
@@ -95,7 +134,7 @@ extern int tidlmask;				/* from pvmd.c */
  **           **
  ***************/
 
-static char rcsid[] = "$Id: tdpro.c,v 1.1 1993/08/30 23:26:52 manchek Exp $";
+static char rcsid[] = "$Id: tdpro.c,v 1.6 1994/10/15 19:37:46 manchek Exp $";
 static char pvmtxt[512];		/* scratch for error log */
 
 int tm_addhost();
@@ -106,17 +145,21 @@ int tm_db();
 int tm_delhost();
 int tm_exit();
 int tm_halt();
+int tm_hoster();
+int tm_hostsync();
 int tm_mca();
 int tm_mstat();
 int tm_notify();
 int tm_pstat();
 int tm_sendsig();
+int tm_sched();
+int tm_setopt();
 int tm_spawn();
 int tm_task();
+int tm_tasker();
 int tm_tickle();
 
 int (*loclswitch[])() = {
-	0,
 	tm_connect,
 	tm_conn2,
 	tm_exit,
@@ -133,6 +176,11 @@ int (*loclswitch[])() = {
 	tm_mca,
 	tm_notify,
 	tm_db,
+	tm_sched,
+	tm_tasker,
+	tm_hoster,
+	tm_hostsync,
+	tm_setopt,
 };
 
 
@@ -153,15 +201,21 @@ char *tm_names[] = {
 	"tm_mca",
 	"tm_notify",
 	"tm_db",
+	"tm_sched",
+	"tm_tasker",
+	"tm_hoster",
+	"tm_hostsync",
+	"tm_setopt",
 };
 
 char *
 tmname(code)
 	int code;
 {
-	if (code < 1 || code > sizeof(tm_names)/sizeof(tm_names[0]))
+	if (code < (int)TM_FIRST || code > (int)TM_LAST)
 		return "unknown";
-	return tm_names[code - 1];
+	code -= TM_FIRST;
+	return tm_names[code];
 }
 
 
@@ -174,7 +228,7 @@ loclentry(tp, mp)
 
 	if (debugmask & PDMMESSAGE) {
 		sprintf(pvmtxt, "loclentry() from t%x code %s\n", mp->m_src,
-				tmname(mp->m_cod));
+				tmname(c));
 		pvmlogerror(pvmtxt);
 /*
 		pvmhdump(mp->m_cfrag->fr_dat, mp->m_cfrag->fr_len, "frag: ");
@@ -188,7 +242,7 @@ loclentry(tp, mp)
 		goto bail;
 	}
 
-	if (c < 1 || c >= sizeof(loclswitch)/sizeof(loclswitch[0])) {
+	if (c < (int)TM_FIRST || c > (int)TM_LAST) {
 		sprintf(pvmtxt, "loclentry() message from t%x with bogus code %d\n",
 				tp->t_tid, c);
 		pvmlogerror(pvmtxt);
@@ -202,6 +256,7 @@ loclentry(tp, mp)
 		goto bail;
 	}
 
+	c -= TM_FIRST;
 	(loclswitch[c])(tp, mp);
 
 bail:
@@ -216,19 +271,6 @@ bail:
  *********************************/
 
 
-#ifdef	NOTMPNAM
-#define	LEN_OF_TMP_NAM	32
-char *pvmtmpnam();
-
-#else	/*NOTMPNAM*/
-#ifdef	L_tmpnam
-#define	LEN_OF_TMP_NAM	L_tmpnam
-#else
-#define	LEN_OF_TMP_NAM	64
-#endif
-
-#endif	/*NOTMPNAM*/
-
 /*	tm_connect()
 *
 *	Task connecting to pvmd phase 1.
@@ -255,6 +297,7 @@ tm_connect(tp, mp)
 	int ver;						/* task's libpvm t-d proto version */
 	char authfn[LEN_OF_TMP_NAM];	/* t-auth file name */
 	int d;
+	int cc;
 
 	if (upkint(mp, &ver) || upkstr(mp, authfn, sizeof(authfn))) {
 		pvmlogerror("tm_connect() bad msg format\n");
@@ -291,19 +334,22 @@ tm_connect(tp, mp)
 		pvmlogperror("tm_connect() can't open t-auth file");
 		goto bail;
 	}
-	(void)write(d, authfn, 1);
+	cc = write(d, authfn, 1);
+	if (cc != 1) {
+		if (cc == -1)
+			pvmlogperror(authfn);
+		pvmlogerror("tm_connect() can't write t-auth file\n");
+	}
 	(void)close(d);
 
 	tp->t_authnam = TALLOC(LEN_OF_TMP_NAM, char, "auth");
-#ifdef	NOTMPNAM
-	(void)pvmtmpnam(tp->t_authnam);
-#else
-	(void)tmpnam(tp->t_authnam);
-#endif
+	(void)TMPNAMFUN(tp->t_authnam);
 
 	if ((tp->t_authfd = open(tp->t_authnam, O_RDONLY|O_CREAT|O_TRUNC, 0600))
 	== -1) {
 		pvmlogperror("tm_connect() can't create d-auth file");
+		PVM_FREE(tp->t_authnam);
+		tp->t_authnam = 0;
 		goto bail;
 	}
 
@@ -349,6 +395,7 @@ bail:
 *		int trccod
 *		int udpmax
 *		int nativecode
+*		string inaddr
 *	}
 */
 
@@ -403,11 +450,12 @@ tm_conn2(tp, mp)
 	*	else make it one
 	*/
 
-	if (tp2 = task_findpid(ipid)) {
+	if ((tp2 = task_findpid(ipid)) && !(tp2->t_flag & (TF_AUTH|TF_CONN))) {
 		if (debugmask & PDMTASK) {
 			sprintf(pvmtxt, "tm_conn2() reconnect task t%x\n", tp2->t_tid);
 			pvmlogerror(pvmtxt);
 		}
+		tp->t_sched = tp2->t_sched;
 
 	} else {
 		if ((tid = tid_new()) < 0) {
@@ -415,6 +463,19 @@ tm_conn2(tp, mp)
 			goto bail;		/* XXX should disconnect nicely */
 		}
 		tp2 = task_new(tid);
+
+		if (pvmschedtid) {
+			/* inform the scheduler about the new task */
+
+			tp->t_sched = pvmschedtid;
+			mp = mesg_new(0);
+			pkint(mp, 1);
+			pkint(mp, tid);
+			mp->m_cod = SM_EXECACK;		/* XXX yecch, overload */
+			mp->m_dst = pvmschedtid;
+			sendmessage(mp);
+		}
+
 		if (debugmask & PDMTASK) {
 			sprintf(pvmtxt, "tm_conn2() new task t%x\n", tp2->t_tid);
 			pvmlogerror(pvmtxt);
@@ -431,9 +492,12 @@ tm_conn2(tp, mp)
 	if (tp2->t_pid != pid)
 		task_setpid(tp2, pid);
 	tp2->t_rxp = tp->t_rxp;
+	tp2->t_sched = tp->t_sched;
 	tp->t_sock = -1;	/* tp will be freed by loclinput() */
 	tp->t_rxp = 0;
 	tp = tp2;
+	if (ipid != pid)
+		tp->t_flag &= ~TF_FORKD;
 
 	/*
 	*	kick it in the butt; it's ready to go
@@ -453,6 +517,7 @@ tm_conn2(tp, mp)
 	pkint(mp, ourudpmtu);
 	pkint(mp, myndf);
 	pkstr(mp, inadport_hex(&(hosts->ht_hosts[hosts->ht_local]->hd_sad)));
+	pkint(mp, pvmschedtid);
 	mp->m_dst = tp->t_tid;
 	mp->m_cod = TM_CONN2;
 	mp->m_flag |= MM_PRIO;
@@ -484,7 +549,8 @@ tm_exit(tp, mp)
 	mp->m_cod = TM_EXIT;
 	tp->t_flag |= TF_CLOSE;
 	sendmessage(mp);
-	task_cleanup(tp);
+	if (!(tp->t_flag & TF_FORKD))
+		task_cleanup(tp);
 	return 0;
 }
 
@@ -553,7 +619,7 @@ tm_pstat(tp, mp)
 /*	tm_addhost()
 *
 *	Task requesting to add hosts to virtual machine.  Exit point is
-*	here or dm_addhostack().
+*	here or dm_addack().
 *
 *	TM_ADDHOST
 *	call {
@@ -561,8 +627,14 @@ tm_pstat(tp, mp)
 *		string names[nhosts]
 *	}
 *	ret {
-*		int nhosts			// or negative for error
-*		int status[nhosts]	// status (tid if successful) of each host
+*		int nhosts			// or error code
+*		int narches			// if nhosts >= 0
+*		{
+*			int tid			// or error code
+*			string name
+*			string arch
+*			int speed
+*		} [nhosts]
 *	}
 */
 
@@ -593,10 +665,11 @@ tm_addhost(tp, mp)
 
 	/* forward message to master pvmd */
 
-	mp->m_dst = hosts->ht_hosts[hosts->ht_master]->hd_hostpart | TIDPVMD;
-	mp->m_cod = DM_ADDHOST;
-	mp->m_wid = wp->wa_wid;
 	mp->m_ref++;
+	mp->m_src = mytid;
+	mp->m_dst = hosts->ht_hosts[hosts->ht_master]->hd_hostpart | TIDPVMD;
+	mp->m_cod = DM_ADD;
+	mp->m_wid = wp->wa_wid;
 	sendmessage(mp);
 	return 0;
 
@@ -638,7 +711,7 @@ free_wait_spawn(wxp)
 /*	tm_spawn()
 *
 *	Task requesting to spawn other tasks.  Exit point for this
-*	request is here or dm_spawnack().
+*	request is here or dm_execack().
 *
 *	TM_SPAWN
 *	call {
@@ -666,77 +739,69 @@ tm_spawn(tp, mp)
 	struct task *tp;
 	struct mesg *mp;
 {
-	char *file = 0;				/* executable */
-	int flags;					/* options */
-	char *where = 0;			/* location from req */
-	int count;					/* num of tasks */
-	int argc;					/* num of args */
-	char **argv = 0;			/* args */
-	int outtid, outcod;			/* stdout dst and code */
-	int trctid, trccod;			/* trace dst and code */
-	int nenv;					/* num of envars */
-	char **env = 0;				/* envars */
-
-	struct waitc *wp;			/* 'seed' waitc */
-	struct waitc_spawn *wxp;	/* extra data */
-	int *vec;					/* result/status vector */
-	struct htab *htp;			/* set of usable hosts */
+	char *where = 0;				/* location from req */
+	struct waitc *wp;				/* 'seed' waitc */
+	struct waitc_spawn *wxp = 0;	/* extra data */
+	struct htab *htp;				/* set of usable hosts */
 	struct hostd *hp;
 	int hh;
 	int i;
 
 	/*
 	* unpack spawn command from task
-	* XXX should clean up this code and upk directly into waitc_spawn
 	*/
 
-	if (upkstralloc(mp, &file)
-	|| upkint(mp, &flags)
+	wxp = TALLOC(1, struct waitc_spawn, "waix");
+	BZERO((char*)wxp, sizeof(struct waitc_spawn));
+
+	if (upkstralloc(mp, &wxp->w_file)
+	|| upkint(mp, &wxp->w_flags)
 	|| upkstralloc(mp, &where)
-	|| upkint(mp, &count)
-	|| upkint(mp, &argc))
+	|| upkint(mp, &wxp->w_veclen)
+	|| upkint(mp, &wxp->w_argc))
 		goto bad;
 
-	if (count < 1)
+	if (wxp->w_veclen < 1)
 		goto bad;
 
-	argc++;
-	BZERO((char*)(argv = TALLOC(argc, char*, "argv")),
-			argc * sizeof(char*));
-	argc--;
-	for (i = 0; i < argc; i++)
-		if (upkstralloc(mp, &argv[i]))
+	wxp->w_argv = TALLOC(wxp->w_argc + 1, char*, "argv");
+	BZERO((char*)wxp->w_argv, (wxp->w_argc + 1) * sizeof(char*));
+	for (i = 0; i < wxp->w_argc; i++)
+		if (upkstralloc(mp, &wxp->w_argv[i]))
 			goto bad;
 
-	if (upkuint(mp, &outtid)
-	|| upkuint(mp, &outcod)
-	|| upkuint(mp, &trctid)
-	|| upkuint(mp, &trccod)
-	|| upkuint(mp, &nenv))
+	if (upkuint(mp, &wxp->w_outtid)
+	|| upkuint(mp, &wxp->w_outcod)
+	|| upkuint(mp, &wxp->w_trctid)
+	|| upkuint(mp, &wxp->w_trccod))
 		goto bad;
-	BZERO((char*)(env = TALLOC((nenv + 1), char*, "env")),
-			(nenv + 1) * sizeof(char*));
-	for (i = 0; i < nenv; i++)
-		if (upkstralloc(mp, &env[i]))
+
+	if (upkuint(mp, &wxp->w_nenv))
+		goto bad;
+	wxp->w_env = TALLOC((wxp->w_nenv + 1), char*, "env");
+	BZERO((char*)wxp->w_env, (wxp->w_nenv + 1) * sizeof(char*));
+	for (i = 0; i < wxp->w_nenv; i++)
+		if (upkstralloc(mp, &wxp->w_env[i]))
 			goto bad;
 
 	/*
 	* make host set containing hosts (matching where option)
 	*/
 
-	if ((flags & (PvmTaskHost|PvmTaskArch)) && !where)
+	if ((wxp->w_flags & (PvmTaskHost|PvmTaskArch)) && !where)
 		goto bad;
 
 	htp = ht_new(1);
 
-	if (flags & PvmTaskHost) {			/* given host */
+	if (wxp->w_flags & PvmTaskHost) {			/* given host */
 		if (hp = nametohost(hosts, where))
 			ht_insert(htp, hp);
 
 	} else {
-		if (flags & PvmTaskArch) {		/* given arch */
+		if (wxp->w_flags & PvmTaskArch) {		/* given arch */
 			for (hh = hosts->ht_last; hh > 0; hh--)
-				if ((hp = hosts->ht_hosts[hh]) && !strcmp(where, hp->hd_arch))
+				if ((hp = hosts->ht_hosts[hh])
+				&& !strcmp(where, hp->hd_arch))
 					ht_insert(htp, hp);
 
 		} else {						/* anywhere */
@@ -744,7 +809,7 @@ tm_spawn(tp, mp)
 		}
 	}
 
-	if (flags & PvmHostCompl) {
+	if (wxp->w_flags & PvmHostCompl) {
 		for (hh = hosts->ht_last; hh > 0; hh--) {
 			if (hh <= htp->ht_last && (hp = htp->ht_hosts[hh]))
 				ht_delete(htp, hp);
@@ -763,31 +828,15 @@ tm_spawn(tp, mp)
 	* assign each task to a host
 	*/
 
+	wxp->w_ht = htp;
+	wxp->w_vec = TALLOC(wxp->w_veclen, int, "vec");
+	BZERO((char*)wxp->w_vec, wxp->w_veclen * sizeof(int));
+	wxp->w_togo = wxp->w_veclen;
+
 	wp = wait_new(WT_SPAWN);
 	wp->wa_tid = tp->t_tid;
-
-	wxp = TALLOC(1, struct waitc_spawn, "waix");
-	BZERO((char*)wxp, sizeof(struct waitc_spawn));
 	wp->wa_spec = (void*)wxp;
-
-	wxp->w_file = file;
-	file = 0;
-	wxp->w_flags = flags;
-	wxp->w_argc = argc;
-	wxp->w_argv = argv;
-	argv = 0;
-	wxp->w_outtid = outtid;
-	wxp->w_outcod = outcod;
-	wxp->w_trctid = trctid;
-	wxp->w_trccod = trccod;
-	wxp->w_nenv = nenv;
-	wxp->w_env = env;
-	env = 0;
-	wxp->w_ht = htp;
-	vec = TALLOC(count, int, "vec");
-	BZERO((char*)vec, count * sizeof(int));
-	wxp->w_vec = vec;
-	wxp->w_togo = wxp->w_veclen = count;
+	wxp = 0;
 
 	assign_tasks(wp);
 
@@ -805,22 +854,10 @@ bad:
 	pvmlogerror(pvmtxt);
 
 cleanup:
-	if (file)
-		PVM_FREE(file);
 	if (where)
 		PVM_FREE(where);
-	if (argv) {
-		for (i = 0; i < argc; i++)
-			if (argv[i])
-				PVM_FREE(argv[i]);
-		PVM_FREE(argv);
-	}
-	if (env) {
-		for (i = 0; i < nenv; i++)
-			if (env[i])
-				PVM_FREE(env[i]);
-		PVM_FREE(env);
-	}
+	if (wxp)
+		free_wait_spawn(wxp);
 	return 0;
 }
 
@@ -848,6 +885,7 @@ assign_tasks(wp)
 	int t;
 	int i;
 	int tid;
+	struct timeval now;
 
 	static int lasthh = -1;		/* for assigning hosts */
 
@@ -883,6 +921,18 @@ assign_tasks(wp)
 		for (t = 0; t < wxp->w_veclen; t++) {
 			tid = wxp->w_vec[t];
 			pkint(mp, tid);
+			if (TIDISTASK(tid) && wxp->w_trctid) {
+				mp2 = mesg_new(0);
+				mp2->m_cod = wxp->w_trccod;
+				mp2->m_dst = wxp->w_trctid;
+				gettimeofday(&now, (struct timezone*)0);
+				pkint(mp2, (int)now.tv_sec);
+				pkint(mp2, (int)now.tv_usec);
+				pkint(mp2, tid);
+				pkint(mp2, TEV_SPNTASK);
+				pkint(mp2, wp->wa_tid);
+				sendmessage(mp2);
+			}
 			if (TIDISTASK(tid) && wxp->w_outtid) {
 				mp2 = mesg_new(0);
 				mp2->m_cod = wxp->w_outcod;
@@ -955,7 +1005,7 @@ assign_tasks(wp)
 			for (i = 0; i < wxp->w_nenv; i++)
 				pkstr(mp, wxp->w_env[i]);
 			mp->m_dst = hp->hd_hostpart | TIDPVMD;
-			mp->m_cod = DM_SPAWN;
+			mp->m_cod = DM_EXEC;
 			mp->m_wid = wp2->wa_wid;
 
 			do {
@@ -1002,10 +1052,11 @@ tm_sendsig(tp, mp)
 		pvmlogerror(pvmtxt);
 		return 0;
 	}
+	mp->m_ref++;
+	mp->m_src = mytid;
 	mp->m_dst = (tid & TIDHOST) | TIDPVMD;
 	mp->m_cod = DM_SENDSIG;
 	mp->m_wid = 0;
-	mp->m_ref++;
 	sendmessage(mp);
 
 	mp = mesg_new(0);
@@ -1026,10 +1077,9 @@ tm_sendsig(tp, mp)
 *		int nhosts
 *		int narches
 *		{
-*			int index
+*			int tid
 *			string name
 *			string arch
-*			int mtu
 *			int speed
 *		} [nhosts]
 *	}
@@ -1095,11 +1145,14 @@ tm_halt(tp, mp)
 *		int where
 *	}
 *	ret {
+*		int errcode
 *		{
 *			int tid
 *			int ptid
 *			int hostpart
 *			int flag
+*			string a_out
+*			int pid
 *		} [count]    // implied
 *	}
 */
@@ -1110,7 +1163,6 @@ tm_task(tp, mp)
 	struct mesg *mp;
 {
 	int where;
-	struct task *tp2;
 	struct mesg *mp2;
 	struct waitc *wp;
 	struct waitc *wp2 = 0;	/* master waitc of peer group */
@@ -1293,6 +1345,8 @@ tm_tickle(tp, mp)
 			sprintf(pvmtxt, "tm_tickle() failing %s\n", hp->hd_name);
 			hostfailentry(hp);
 			ht_delete(hosts, hp);
+			if (newhosts)
+				ht_delete(newhosts, hp);
 			pkint(mp, 1);
 		} else {
 			if (hp)
@@ -1372,10 +1426,11 @@ tm_delhost(tp, mp)
 
 	/* forward message to master pvmd */
 
+	mp->m_ref++;
+	mp->m_src = mytid;
 	mp->m_dst = hosts->ht_hosts[hosts->ht_master]->hd_hostpart | TIDPVMD;
 	mp->m_cod = DM_DELHOST;
 	mp->m_wid = wp->wa_wid;
-	mp->m_ref++;
 	sendmessage(mp);
 	return 0;
 
@@ -1530,19 +1585,11 @@ tm_notify(tp, mp)
 	struct waitc *wp;
 	int i;
 
-	if (upkint(mp, &what) || upkint(mp, &code) || upkint(mp, &count)
-	|| count < 0) {
+	if (upkint(mp, &what) || upkint(mp, &code) || upkint(mp, &count)) {
 		pvmlogerror("tm_notify() bad msg format\n");
 		return 0;
 	}
 
-/*
-	mp2 = mesg_new(0);
-	mp2->m_dst = tp->t_tid;
-	mp2->m_cod = TM_NOTIFY;
-	sendmessage(mp2);
-*/
-
 	switch (what) {
 
 	case PvmTaskExit:
@@ -1608,10 +1655,23 @@ dead:
 		break;
 
 	case PvmHostAdd:
-		wp = wait_new(WT_HOSTA);
-		wp->wa_tid = tp->t_tid;
-		wp->wa_on = tp->t_tid;
-		wp->wa_dep = code;
+	/* find existing waitc to update or create new one */
+
+		for (wp = waitlist->wa_link; wp != waitlist; wp = wp->wa_link)
+			if (wp->wa_kind == WT_HOSTA && wp->wa_tid == tp->t_tid)
+				break;
+		if (wp = waitlist) {
+			wp = wait_new(WT_HOSTA);
+			wp->wa_tid = tp->t_tid;
+			wp->wa_on = tp->t_tid;
+		}
+		if (count) {
+			wp->wa_dep = code;
+			wp->wa_count = count;
+
+		} else {
+			wait_delete(wp);
+		}
 		break;
 
 	default:
@@ -1706,6 +1766,7 @@ tm_db(tp, mp)
 	wp->wa_on = hosts->ht_hosts[hosts->ht_master]->hd_hostpart;
 
 	mp->m_ref++;
+	mp->m_src = mytid;
 	mp->m_dst = hosts->ht_hosts[hosts->ht_master]->hd_hostpart | TIDPVMD;
 	mp->m_cod = DM_DB;
 	mp->m_wid = wp->wa_wid;
@@ -1714,3 +1775,366 @@ tm_db(tp, mp)
 }
 
 
+/*	tm_sched()
+*
+*	Task wants to register as scheduler.
+*
+*	TM_SCHED
+*	call {
+*	}
+*	ret {
+*		int oldschedtid
+*		int tid			// of master pvmd
+*		string name
+*		string arch
+*		int speed
+*	}
+*/
+
+int
+tm_sched(tp, mp)
+	struct task *tp;
+	struct mesg *mp;
+{
+	struct mesg *mp2;
+	struct hostd *hp;
+
+	mp2 = mesg_new(0);
+	mp2->m_dst = tp->t_tid;
+	mp2->m_cod = TM_SCHED;
+	pkint(mp2, pvmschedtid);
+	hp = hosts->ht_hosts[hosts->ht_master];
+	pkint(mp2, hp->hd_hostpart);
+	pkstr(mp2, hp->hd_name);
+	pkstr(mp2, hp->hd_arch ? hp->hd_arch : "");
+	pkint(mp2, hp->hd_speed);
+	sendmessage(mp2);
+
+	if (debugmask & PDMSCHED) {
+		sprintf(pvmtxt, "tm_sched() old t%x new t%x\n", pvmschedtid, tp->t_tid);
+		pvmlogerror(pvmtxt);
+	}
+	pvmschedtid = tp->t_tid;
+	tp->t_flag |= TF_ISSCHED;
+	return 0;
+}
+
+
+/*	tm_tasker()
+*
+*	Task registers as local task starter.
+*
+*	TM_TASKER
+*	call {
+*		int regme	// true to register, false to unregister
+*	}
+*	ret {
+*		int code	// 0 if okay, else error
+*	}
+*/
+
+int
+tm_tasker(tp, mp)
+	struct task *tp;
+	struct mesg *mp;
+{
+	int regme;
+	struct mesg *mp2;
+
+	upkint(mp, &regme);
+	mp2 = mesg_new(0);
+	mp2->m_dst = mp->m_src;
+	mp2->m_cod = mp->m_cod;
+	if (regme) {
+		if (taskertid) {
+			pkint(mp2, PvmAlready);
+
+		} else {
+			taskertid = tp->t_tid;
+			tp->t_flag |= TF_ISTASKER;
+			if (debugmask & PDMTASK) {
+				sprintf(pvmtxt, "tm_tasker() register t%x \n", tp->t_tid);
+				pvmlogerror(pvmtxt);
+			}
+			pkint(mp2, 0);
+		}
+
+	} else {
+		if (taskertid == tp->t_tid) {
+			taskertid = 0;
+			tp->t_flag &= ~TF_ISTASKER;
+			if (debugmask & PDMTASK) {
+				sprintf(pvmtxt, "tm_tasker() unregister t%x \n", tp->t_tid);
+				pvmlogerror(pvmtxt);
+			}
+			pkint(mp2, 0);
+
+		} else {
+			if (debugmask & PDMTASK) {
+				sprintf(pvmtxt, "tm_tasker() t%x tries to unregister?\n",
+						tp->t_tid);
+				pvmlogerror(pvmtxt);
+			}
+			pkint(mp2, PvmNoTask);
+		}
+	}
+	sendmessage(mp2);
+	return 0;
+}
+
+
+/*	tm_hoster()
+*
+*	Task registers as slave pvmd starter.
+*
+*	TM_HOSTER
+*	call {
+*	}
+*	ret {
+*	}
+*/
+
+int
+tm_hoster(tp, mp)
+	struct task *tp;
+	struct mesg *mp;
+{
+	struct mesg *mp2;
+
+	hostertid = tp->t_tid;
+	tp->t_flag |= TF_ISHOSTER;
+	mp2 = mesg_new(0);
+	mp2->m_dst = mp->m_src;
+	mp2->m_cod = mp->m_cod;
+	sendmessage(mp2);
+	if (debugmask & PDMSTARTUP) {
+		sprintf(pvmtxt, "tm_hoster() register t%x \n", tp->t_tid);
+		pvmlogerror(pvmtxt);
+	}
+	return 0;
+}
+
+
+/*	tm_hostsync()
+*
+*	Task wants clock sample from host.
+*
+*	TM_HOSTSYNC
+*	call {
+*		int where	// pvmd tid
+*	}
+*	ret {
+*		int sec		// time of day clock
+*		int usec
+*	}
+*/
+
+int
+tm_hostsync(tp, mp)
+	struct task *tp;
+	struct mesg *mp;
+{
+	int where;
+	struct hostd *hp;
+	struct mesg *mp2;
+	struct waitc *wp;
+
+	if (upkuint(mp, &where)) {
+		pvmlogerror("tm_hostsync() bad msg format\n");
+		return 0;
+	}
+
+	mp2 = mesg_new(0);
+	mp2->m_dst = mp->m_src;
+	mp2->m_cod = mp->m_cod;
+
+	if (!(hp = tidtohost(hosts, where))) {
+		pkint(mp2, PvmNoHost);
+		sendmessage(mp2);
+		return 0;
+	}
+
+	wp = wait_new(WT_HOSTSYNC);
+	wp->wa_tid = mp->m_src;
+	wp->wa_on = hp->hd_hostpart;
+	wp->wa_mesg = mp2;
+
+	mp2 = mesg_new(0);
+	mp2->m_dst = where | TIDPVMD;
+	mp2->m_cod = DM_HOSTSYNC;
+	mp2->m_wid = wp->wa_wid;
+
+	sendmessage(mp2);
+	return 0;
+}
+
+
+/*	tm_setopt()
+*
+*	Set task parameters
+*
+*	TM_SETOPT
+*	call {
+*		{
+*			int what		// which option
+*			string value	// option value
+*		} []				// implied
+*	}
+*	ret {
+*/
+
+int
+tm_setopt(tp, mp)
+	struct task *tp;
+	struct mesg *mp;
+{
+	int what;
+	char *val;
+	int x;
+
+	while (!upkint(mp, &what) && !upkstralloc(mp, &val)) {
+		switch (what) {
+
+		case TS_OUTTID:
+			x = pvmxtoi(val);
+			change_output(tp, x, tp->t_outcod);
+			break;
+
+		case TS_OUTCOD:
+			x = pvmxtoi(val);
+			change_output(tp, tp->t_outtid, x);
+			break;
+
+		case TS_TRCTID:
+			x = pvmxtoi(val);
+			change_trace(tp, x, tp->t_trccod);
+			break;
+
+		case TS_TRCCOD:
+			x = pvmxtoi(val);
+			change_trace(tp, tp->t_trctid, x);
+			break;
+
+		default:
+			if (strlen(val) > sizeof(pvmtxt)-50)
+				val[sizeof(pvmtxt)-50] = 0;
+			sprintf(pvmtxt, "tm_setopt() ? option %d val <%s>\n", what, val);
+			pvmlogerror(pvmtxt);
+			break;
+		}
+		if (val)
+			PVM_FREE(val);
+	}
+	mp = mesg_new(0);
+	mp->m_dst = tp->t_tid;
+	mp->m_cod = TM_SETOPT;
+	sendmessage(mp);
+	return 0;
+}
+
+
+int
+change_output(tp, outtid, outcod)
+	struct task *tp;
+	int outtid, outcod;
+{
+	struct mesg *mp;
+
+	if (tp->t_outtid != outtid || tp->t_outcod != outcod) {
+		if (tp->t_outtid) {
+			mp = mesg_new(0);
+			mp->m_cod = tp->t_outcod;
+			mp->m_dst = tp->t_outtid;
+			pkint(mp, tp->t_tid);
+			pkint(mp, 0);
+			sendmessage(mp);
+		}
+		if (debugmask & PDMTASK) {
+			sprintf(pvmtxt, "t%x changes output from <t%x %d> to <t%x %d>\n",
+					tp->t_tid, tp->t_outtid, tp->t_outcod, outtid, outcod);
+			pvmlogerror(pvmtxt);
+		}
+		tp->t_outtid = outtid;
+		tp->t_outcod = outcod;
+		if (tp->t_outtid) {
+			mp = mesg_new(0);
+			mp->m_cod = tp->t_outcod;
+			mp->m_dst = tp->t_outtid;
+			pkint(mp, tp->t_tid);
+			pkint(mp, -1);
+			pkint(mp, tp->t_ptid);
+			sendmessage(mp);
+
+			mp = mesg_new(0);
+			mp->m_cod = tp->t_outcod;
+			mp->m_dst = tp->t_outtid;
+			pkint(mp, tp->t_tid);
+			pkint(mp, -2);
+			pkint(mp, tp->t_ptid);
+			sendmessage(mp);
+		}
+	}
+	return 0;
+}
+
+
+int
+change_trace(tp, trctid, trccod)
+	struct task *tp;
+	int trctid, trccod;
+{
+	struct timeval now;
+	struct mesg *mp;
+
+	if (tp->t_trctid != trctid || tp->t_trccod != trccod) {
+		gettimeofday(&now, (struct timezone*)0);
+		if (tp->t_trctid) {
+			mp = mesg_new(0);
+			mp->m_cod = tp->t_trccod;
+			mp->m_dst = tp->t_trctid;
+			pkint(mp, (int)now.tv_sec);
+			pkint(mp, (int)now.tv_usec);
+			pkint(mp, tp->t_tid);
+			pkint(mp, TEV_ENDTASK);
+			pkint(mp, tp->t_status);
+			pkint(mp, (int)tp->t_utime.tv_sec);
+			pkint(mp, (int)tp->t_utime.tv_usec);
+			pkint(mp, (int)tp->t_stime.tv_sec);
+			pkint(mp, (int)tp->t_stime.tv_usec);
+			sendmessage(mp);
+		}
+		if (debugmask & PDMTASK) {
+			sprintf(pvmtxt, "t%x changes trace from <t%x %d> to <t%x %d>\n",
+					tp->t_tid, tp->t_trctid, tp->t_trccod, trctid, trccod);
+			pvmlogerror(pvmtxt);
+		}
+		tp->t_trctid = trctid;
+		tp->t_trccod = trccod;
+		if (tp->t_trctid) {
+			mp = mesg_new(0);
+			mp->m_cod = tp->t_trccod;
+			mp->m_dst = tp->t_trctid;
+			pkint(mp, (int)now.tv_sec);
+			pkint(mp, (int)now.tv_usec);
+			pkint(mp, tp->t_tid);
+			pkint(mp, TEV_SPNTASK);
+			pkint(mp, tp->t_ptid);
+			sendmessage(mp);
+
+			mp = mesg_new(0);
+			mp->m_cod = tp->t_trccod;
+			mp->m_dst = tp->t_trctid;
+			pkint(mp, (int)now.tv_sec);
+			pkint(mp, (int)now.tv_usec);
+			pkint(mp, tp->t_tid);
+			pkint(mp, TEV_NEWTASK);
+			pkint(mp, tp->t_ptid);
+			pkint(mp, 0);	/* XXX lie */
+			pkstr(mp, tp->t_a_out ? tp->t_a_out : "");
+			sendmessage(mp);
+		}
+	}
+	return 0;
+}
+
+
