@@ -1,6 +1,6 @@
 
 /*
- *           PVM 3.2:  Parallel Virtual Machine System 3.2
+ *         PVM version 3.3:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
@@ -22,8 +22,8 @@
  * purpose.  This software is provided ``as is'' without express or
  * implied warranty.
  *
- * PVM 3.2 was funded in part by the U.S. Department of Energy, the
- * National Science Foundation and the State of Tennessee.
+ * PVM version 3 was funded in part by the U.S. Department of Energy,
+ * the National Science Foundation and the State of Tennessee.
  */
 
 /*
@@ -32,12 +32,25 @@
  *	Wait context descriptors.
  *
 $Log: waitc.c,v $
+ * Revision 1.3  1994/07/18  19:25:18  manchek
+ * wait_get() checks hostpart, ignoring TIDPVMD.
+ * added create/free logging
+ *
+ * Revision 1.2  1994/06/03  20:38:32  manchek
+ * version 3.3.0
+ *
  * Revision 1.1  1993/08/30  23:26:52  manchek
  * Initial revision
  *
  */
 
 #include <stdio.h>
+#ifdef IMA_BSD386
+#include <machine/endian.h>
+#endif
+#ifdef IMA_LINUX
+#include <endian.h>
+#endif
 #ifdef IMA_TITN
 #include <bsd/sys/types.h>
 #else
@@ -47,7 +60,13 @@ $Log: waitc.c,v $
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
+#ifdef	SYSVSTR
+#include <string.h>
+#else
+#include <strings.h>
+#endif
 #include "global.h"
+#include "ddpro.h"
 #include "protoglarp.h"
 #include "pvmalloc.h"
 #include "mesg.h"
@@ -64,6 +83,7 @@ $Log: waitc.c,v $
 extern void pvmbailout();
 char *dmname();
 
+extern int debugmask;				/* from pvmd.c */
 extern struct htab *hosts;			/* from pvmd.c */
 extern int myhostpart;				/* from pvmd.c */
 extern int tidlmask;				/* from pvmd.c */
@@ -76,12 +96,13 @@ struct waitc *waitlist = 0;
  **           **
  ***************/
 
-static char rcsid[] = "$Id: waitc.c,v 1.1 1993/08/30 23:26:52 manchek Exp $";
+static char rcsid[] = "$Id: waitc.c,v 1.3 1994/07/18 19:25:18 manchek Exp $";
 static char pvmtxt[512];				/* scratch for error log */
 static char *waitnames[] = {
 	"addhost", "spawn", "hoststart", "task",
 	"delhost", "htupd", "pstat", "taskx",
-	"mstat", "db", "hostf", "hosta"
+	"mstat", "db", "hostf", "hosta", "hostsync",
+	"taskstart"
 };
 
 
@@ -176,9 +197,14 @@ wait_new(kind)
 
 	wp2->wa_on = wp2->wa_tid = wp2->wa_dep = 0;
 	wp2->wa_mesg = 0;
+	wp2->wa_count = 0;
 	wp2->wa_spec = 0;
 
 	LISTPUTBEFORE(wp, wp2, wa_link, wa_rlink);
+	if (debugmask & PDMWAITC) {
+		pvmlogerror("wait_new():\n");
+		wait_dump(wp2);
+	}
 	return wp2;
 }
 
@@ -212,6 +238,10 @@ void
 wait_delete(wp)
 	struct waitc *wp;
 {
+	if (debugmask & PDMWAITC) {
+		pvmlogerror("wait_delete():\n");
+		wait_dump(wp);
+	}
 	if (wp->wa_mesg)
 		mesg_unref(wp->wa_mesg);
 
@@ -248,7 +278,7 @@ wait_get(hp, mp, kind)
 		pvmlogerror(pvmtxt);
 		return 0;
 	}
-	if (hp && hp->hd_hostpart != wp->wa_on) {
+	if (hp && hp->hd_hostpart != (wp->wa_on & TIDHOST)) {
 		sprintf(pvmtxt, "waitc_get() cod %s from t%x wid %d wrong host\n",
 				dmname(mp->m_cod), mp->m_src, mp->m_wid);
 		pvmlogerror(pvmtxt);
@@ -285,7 +315,8 @@ wait_dump(wp)
 		p += strlen(p);
 		sprintf(p, " %d", wp2->wa_wid);
 	}
-	strcat(p, " }\n");
+	p += strlen(p);
+	sprintf(p, " } cnt %d\n", wp->wa_count);
 	pvmlogerror(pvmtxt);
 }
 
