@@ -1,6 +1,6 @@
 
 /*
- *           PVM 3.2:  Parallel Virtual Machine System 3.2
+ *         PVM version 3.3:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
@@ -22,8 +22,8 @@
  * purpose.  This software is provided ``as is'' without express or
  * implied warranty.
  *
- * PVM 3.2 was funded in part by the U.S. Department of Energy, the
- * National Science Foundation and the State of Tennessee.
+ * PVM version 3 was funded in part by the U.S. Department of Energy,
+ * the National Science Foundation and the State of Tennessee.
  */
 
 /*
@@ -32,6 +32,9 @@
  *	Wait context descriptors.
  *
 $Log: waitc.h,v $
+ * Revision 1.2  1994/06/03  20:38:33  manchek
+ * version 3.3.0
+ *
  * Revision 1.1  1993/08/30  23:26:52  manchek
  * Initial revision
  *
@@ -52,6 +55,7 @@ struct waitc {
 	struct waitc *wa_rpeer;
 /* wait kind-specific stuff below */
 	struct mesg *wa_mesg;		/* message being built */
+	int wa_count;				/* number of times to trigger */
 	void *wa_spec;				/* for kind-specific data */
 };
 
@@ -59,7 +63,7 @@ struct waitc {
 
 #define	WT_ADDHOST		1	/* hosts being added for us */
 #define	WT_SPAWN		2	/* tasks being spawned for us */
-#define	WT_HOSTSTART	3	/* pvmd' starting slaves for us */
+#define	WT_HOSTSTART	3	/* hoster starting slaves for us */
 #define	WT_TASK			4	/* getting task list from other pvmds */
 #define	WT_DELHOST		5	/* hosts being deleted for us */
 #define	WT_HTUPD		6	/* host table update waiting to commit */
@@ -69,6 +73,8 @@ struct waitc {
 #define	WT_DB			10	/* name-server query */
 #define	WT_HOSTF		11	/* waiting for host to fail */
 #define	WT_HOSTA		12	/* waiting for hosts to be added */
+#define	WT_HOSTSYNC		13	/* get remote time of day clock */
+#define	WT_TASKSTART	14	/* tasker starting task */
 
 void wait_init __ProtoGlarp__(( void ));
 struct waitc *wait_new __ProtoGlarp__(( int kind ));
@@ -95,3 +101,8 @@ struct waitc_spawn {
 	char **w_env;				/* envar list */
 };
 
+struct waitc_add {
+	int w_num;				/* number of hosts */
+	struct hostd **w_hosts;	/* vector of host entries */
+};
+
