@@ -1,11 +1,11 @@
 
 /*
- *           PVM 3.2:  Parallel Virtual Machine System 3.2
+ *         PVM version 3.3:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
  *      Authors:  A. L. Beguelin, J. J. Dongarra, G. A. Geist,
- *          R. J. Manchek, B. K. Moore, and V. S. Sunderam
+ *    W. C. Jiang, R. J. Manchek, B. K. Moore, and V. S. Sunderam
  *                   (C) 1992 All Rights Reserved
  *
  *                              NOTICE
@@ -22,8 +22,8 @@
  * purpose.  This software is provided ``as is'' without express or
  * implied warranty.
  *
- * PVM 3.2 was funded in part by the U.S. Department of Energy, the
- * National Science Foundation and the State of Tennessee.
+ * PVM version 3 was funded in part by the U.S. Department of Energy,
+ * the National Science Foundation and the State of Tennessee.
  */
 
 /*
@@ -88,7 +88,7 @@ main(argc, argv)
 	printf("i'm t%x\n", mytid);
 */
 
-	pvm_config(&nprocessors, (int*)0, (struct hostinfo**)0);
+	pvm_config(&nprocessors, (int*)0, (struct pvmhostinfo**)0);
 	if (nprocessors > wd)
 		nprocessors = wd;
 	fprintf(stderr, "using %d tile servers in calculation\n", nprocessors);
