@@ -1,11 +1,11 @@
 
 /*
- *           PVM 3.0:  Parallel Virtual Machine System 3.0
+ *         PVM version 3.3:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
  *      Authors:  A. L. Beguelin, J. J. Dongarra, G. A. Geist,
- *          R. J. Manchek, B. K. Moore, and V. S. Sunderam
+ *    W. C. Jiang, R. J. Manchek, B. K. Moore, and V. S. Sunderam
  *                   (C) 1992 All Rights Reserved
  *
  *                              NOTICE
@@ -22,8 +22,8 @@
  * purpose.  This software is provided ``as is'' without express or
  * implied warranty.
  *
- * PVM 3.0 was funded in part by the U.S. Department of Energy, the
- * National Science Foundation and the State of Tennessee.
+ * PVM version 3 was funded in part by the U.S. Department of Energy,
+ * the National Science Foundation and the State of Tennessee.
  */
 
 /*
@@ -47,6 +47,7 @@
 #include <X11/Xaw/Toggle.h>
 #include <X11/Xaw/Form.h>
 #include "pvm3.h"
+#include "../src/bfunc.h"
 #include "bars.xbm"
 #include "neww.xbm"
 #include "quit.xbm"
@@ -67,6 +68,7 @@
 #define	byteswide(w) (((w)+7)/8)
 #define	TILEHEIGHT	16
 
+typedef	unsigned int IBIT32;
 
 /* describes an image canvas (currently only one) */
 
@@ -126,9 +128,9 @@ int blueShift;
 Colormap xCmap;
 int cmapInd[NCMV];
 Visual *defVis;
-u_long fclutr[256];		/* pseudo-color lut for true-color display */
-u_long fclutg[256];
-u_long fclutb[256];
+IBIT32 fclutr[256];		/* pseudo-color lut for true-color display */
+IBIT32 fclutg[256];
+IBIT32 fclutb[256];
 GC rubGc;				/* rubberbox gc */
 GC canGc;				/* canvas painting gc */
 Cursor crossCr;
@@ -184,6 +186,7 @@ typedef struct {
 	Bool mono;			/* force monochrome display */
 	String worker;		/* worker a.out name */
 	Bool bars;			/* display processor id bars */
+	int n;				/* number of workers */
 } app_res_t, *app_resp_t;
 
 static app_res_t app_res;
@@ -195,6 +198,8 @@ static XtResource res_list[] = {
 		XtOffset(app_resp_t, mono), XtRString, "off" },
 	{ "bars", "Bars", XtRBool, sizeof(Bool),
 		XtOffset(app_resp_t, bars), XtRString, "on" },
+	{ "nWorkers", "NWorkers", XtRInt, sizeof(int),
+		XtOffset(app_resp_t, n), XtRString, "-1" },
 };
 
 static XrmOptionDescRec knownargs[] = {
@@ -203,6 +208,7 @@ static XrmOptionDescRec knownargs[] = {
 	{ "-worker", ".worker", XrmoptionSepArg, 0 },
 	{ "-bars", ".bars", XrmoptionNoArg, "on" },
 	{ "+bars", ".bars", XrmoptionNoArg, "off" },
+	{ "-n", ".nWorkers", XrmoptionSepArg, 0 },
 };
 
 static XtCallbackRec callback[2] = { { 0, 0 }, { 0, 0 } };
@@ -245,6 +251,7 @@ main(argc, argv)
 		fputs("  -worker filename  set calculation task name\n", stderr);
 		fputs("  -/+mono           set/unset forced mono mode\n", stderr);
 		fputs("  -/+bars           display/don't display processor bars\n", stderr);
+		fputs("  -n count          set number of workers\n", stderr);
 		exit(1);
 	}
 
@@ -295,7 +302,13 @@ start_workers()
 	int n;		/* actual number started */
 	int i;
 
-	pvm_config(&nserv, (int*)0, (struct hostinfo**)0);
+	pvm_config(&nserv, (int*)0, (struct pvmhostinfo**)0);
+	if (app_res.n > 0 && app_res.n <= nserv * 10)
+		nserv = app_res.n;
+/*
+	if (nserv > 1)
+		nserv--;
+*/
 
 	servtids = TALLOC(nserv, int);
 	servflist = TALLOC(nserv, int);
@@ -362,8 +375,8 @@ setup_color(mono)
 {
 	int i, j;
 	XColor colr;
-	u_long rbowr[NCMV], rbowg[NCMV], rbowb[NCMV];
-	u_long mbo = 0x04030201;	/* to test machine byte order */
+	IBIT32 rbowr[NCMV], rbowg[NCMV], rbowb[NCMV];
+	IBIT32 mbo = 0x04030201;	/* to test machine byte order */
 
 	nPlanes = DefaultDepth(xDisp, xScrn);
 	defVis = DefaultVisual(xDisp, xScrn);
@@ -441,7 +454,7 @@ forcebw:
 */
 
 mkrbow(r, g, b, n)
-	u_long *r, *g, *b;	/* red, grn, blu lut return */
+	IBIT32 *r, *g, *b;	/* red, grn, blu lut return */
 	int n;				/* number of entries (length of r, g, b) */
 {
 	int i, j;
@@ -604,7 +617,7 @@ create_xep_widget()
 	XtSetArg(args[n], XtNright, XtChainLeft); n++;
 	XtSetArg(args[n], XtNtop, XtChainTop); n++;
 	XtSetArg(args[n], XtNbottom, XtChainTop); n++;
-	callback[0].callback = quit_cb;
+	callback[0].callback = (XtCallbackProc)quit_cb;
 	callback[0].closure = (caddr_t)0;
 	XtSetArg(args[n], XtNcallback, callback); n++;
 	pm = XCreatePixmapFromBitmapData(xDisp, xRootW,
@@ -618,7 +631,7 @@ create_xep_widget()
 	XtSetArg(args[n], XtNright, XtChainLeft); n++;
 	XtSetArg(args[n], XtNtop, XtChainTop); n++;
 	XtSetArg(args[n], XtNbottom, XtChainTop); n++;
-	callback[0].callback = workers_cb;
+	callback[0].callback = (XtCallbackProc)workers_cb;
 	callback[0].closure = (caddr_t)0;
 	XtSetArg(args[n], XtNcallback, callback); n++;
 	XtSetArg(args[n], XtNfromHoriz, w); n++;
@@ -633,7 +646,7 @@ create_xep_widget()
 	XtSetArg(args[n], XtNright, XtChainLeft); n++;
 	XtSetArg(args[n], XtNtop, XtChainTop); n++;
 	XtSetArg(args[n], XtNbottom, XtChainTop); n++;
-	callback[0].callback = redo_cb;
+	callback[0].callback = (XtCallbackProc)redo_cb;
 	callback[0].closure = (caddr_t)0;
 	XtSetArg(args[n], XtNcallback, callback); n++;
 	XtSetArg(args[n], XtNfromHoriz, w); n++;
@@ -648,7 +661,7 @@ create_xep_widget()
 	XtSetArg(args[n], XtNright, XtChainLeft); n++;
 	XtSetArg(args[n], XtNtop, XtChainTop); n++;
 	XtSetArg(args[n], XtNbottom, XtChainTop); n++;
-	callback[0].callback = zout_cb;
+	callback[0].callback = (XtCallbackProc)zout_cb;
 	callback[0].closure = (caddr_t)0;
 	XtSetArg(args[n], XtNcallback, callback); n++;
 	XtSetArg(args[n], XtNfromHoriz, w); n++;
@@ -663,7 +676,7 @@ create_xep_widget()
 	XtSetArg(args[n], XtNright, XtChainLeft); n++;
 	XtSetArg(args[n], XtNtop, XtChainTop); n++;
 	XtSetArg(args[n], XtNbottom, XtChainTop); n++;
-	callback[0].callback = bars_cb;
+	callback[0].callback = (XtCallbackProc)bars_cb;
 	callback[0].closure = (caddr_t)0;
 	XtSetArg(args[n], XtNcallback, callback); n++;
 	XtSetArg(args[n], XtNfromHoriz, w); n++;
@@ -792,7 +805,7 @@ canvas_ev(wgt, cli, xev, rem)
 		b1 = imCan.cn_dat + max(0, ox) + max(0, oy) * imCan.cn_wd;
 		b2 = dat + max(0, -ox) + max(0, -oy) * wd;
 		while (h-- > 0) {
-			bcopy(b1, b2, w);
+			BCOPY(b1, b2, w);
 			b1 += imCan.cn_wd;
 			b2 += wd;
 		}
@@ -995,12 +1008,12 @@ rubbox(w, x1, y1, x2, y2)
 
 /*	sw4()
 *
-*	Byteswap a long integer.
+*	Byteswap a 32-bit integer.
 */
 
-u_long
+IBIT32
 sw4(x)
-	u_long x;
+	IBIT32 x;
 {
 	u_char *cp = (u_char*)&x;
 	u_char c;
@@ -1031,8 +1044,8 @@ repaint_region(cnp, x1, y1, x2, y2)
 	int xa, ya;
 	unsigned char *sa;
 	char *da;
-	u_long pixv;
-	u_long pixr, pixg, pixb;
+	IBIT32 pixv;
+	IBIT32 pixr, pixg, pixb;
 	int lbs;
 	int lbm;
 
@@ -1088,7 +1101,7 @@ repaint_region(cnp, x1, y1, x2, y2)
 				dx = (x1 - cnp->cn_ox) * cnp->cn_zoom;
 				xa = 0;
 				sa = ribuf + sy * cnp->cn_wd + sx;
-				da = ximbuf + dy * cnp->cn_xim->bytes_per_line + dx * sizeof(long);
+				da = ximbuf + dy * cnp->cn_xim->bytes_per_line + dx * sizeof(IBIT32);
 				while (sx < x2 && dx < cnp->cn_wd) {
 					pixv = lbm & *sa;
 					pixr = fclutr[pixv];
@@ -1098,11 +1111,11 @@ repaint_region(cnp, x1, y1, x2, y2)
 							| (greenMask & (pixg << greenShift))
 							| (blueMask & (pixb << blueShift));
 					if (revByte)
-						*((u_long*)da) = sw4(pixv);
+						*((IBIT32*)da) = sw4(pixv);
 					else
-						*((u_long*)da) = pixv;
+						*((IBIT32*)da) = pixv;
 		
-					da += sizeof(long);
+					da += sizeof(IBIT32);
 					dx++;
 					if (++xa >= cnp->cn_zoom) {
 						xa = 0;
@@ -1250,10 +1263,14 @@ start_recalc()
 		printf("sent job %d to serv %d: %dx%d %f,%f/%f,%f\n",
 				jobnext, servfree,
 				wdht[0], wdht[1], reim[0], reim[1], reim[2], reim[3]);
-*/
 		pvm_initsend(PvmDataDefault);
 		pvm_pkdouble(reim, 4, 1);
 		pvm_pkint(wdht, 2, 1);
+*/
+/*
+		pvm_packf("%+ %4lf %2d", PvmDataDefault, reim, wdht);
+*/
+		pvm_packf("%+ %2lx %2d", PvmDataDefault, reim, wdht);
 /*
 		umbuf_dump(pvm_getsbuf());
 */
@@ -1315,7 +1332,10 @@ claim_tile()
 	y1 = (servjobs[s] * ht) / njobs;
 	y2 = ((servjobs[s] + 1) * ht) / njobs;
 	h = y2 - y1;
+/*
 	pvm_upkbyte((char*)imCan.cn_dat + y1 * wd, wd * h, 1);
+*/
+	pvm_unpackf("%*c", wd * h, (char*)imCan.cn_dat + y1 * wd);
 
 	if (dobars)
 		label_row(y1, y2, s);
@@ -1350,10 +1370,15 @@ claim_tile()
 		printf("sent job %d to serv %d: %dx%d %f,%f/%f,%f\n",
 				jobnext, servfree,
 				wdht[0], wdht[1], reim[0], reim[1], reim[2], reim[3]);
-*/
 		pvm_initsend(PvmDataDefault);
 		pvm_pkdouble(reim, 4, 1);
 		pvm_pkint(wdht, 2, 1);
+*/
+/*
+		pvm_packf("%+ %4lf %2d", PvmDataDefault, reim, wdht);
+*/
+		pvm_packf("%+ %2lx %2d", PvmDataDefault, reim, wdht);
+
 		if (pvm_send(servtids[servfree], 1)) {
 			pvm_exit();
 			exit(1);
@@ -1376,10 +1401,11 @@ label_row(y1, y2, n)
 	int h = y2 - y1;
 	int y, x;
 	u_char *ba;
+	int c = isMono ? 128 : 240;
 
 	ba = imCan.cn_dat + y1 * wd;
 	for (y = h; y-- > 0; ) {
-		bzero(ba, 21);
+		BZERO(ba, 21);
 		ba += wd;
 	}
 	ba = imCan.cn_dat + y1 * wd;
@@ -1387,7 +1413,7 @@ label_row(y1, y2, n)
 		ba += wd;
 		for (x = 0; x < 20; x++)
 			if (x & 3)
-				ba[x] = (n & (1 << x / 4)) ? 255 : 240;
+				ba[x] = (n & (1 << x / 4)) ? 255 : c;
 	}
 	repaint_region(&imCan, 0, y1, 20, y2 - 1);
 	refresh_region(&imCan, 0, y1, 20, y2 - 1);
